{*******************************************************}
{                                                       }
{       IBAccess                                        }
{       Backup database dialog                          }
{                                                       }
{       Author:                                         }
{       Toni Martir                                     }
{       http://www.ibaccess.org                         }
{       e-mail: toni@pala.com                           }
{                                                       }
{       OpenSource license (MPL)                        }
{                                                       }
{*******************************************************}

unit backupdatabase;

interface

uses
  SysUtils, Classes, QForms, QDialogs,
  diaser, Db,variants,
  rpibdb,ibacconst, ibman, IBServices,ibmanst,ibproces,
 dbclient, QMask, QStdCtrls, QControls, QGrids, QDBGrids, QButtons;

type
  TFBackup = class(TFService)
    DataSource1: TDataSource;
    DBGrid1: TDBGrid;
    DMemtable: TClientdataset;
    DMemtableFile: TStringField;
    DMemtableSize: TIntegerField;
    Label4: TLabel;
    GRestore: TGroupBox;
    RadioService: TRadioButton;
    RadioCommandline: TRadioButton;
    CheckService: TCheckBox;
    DMemtableUnit: TStringField;
    IbMan1: TIbManager;
    Erole: TEdit;
    Label5: TLabel;
    BShowops: TButton;
    BHideOps: TButton;
    GOptions: TGroupBox;
    CheckIgnore: TCheckBox;
    Checktrans: TCheckBox;
    Checkmeta: TCheckBox;
    CheckNoGarbage: TCheckBox;
    CheckOldMeta: TCheckBox;
    CheckNotrans: TCheckBox;
    CheckConvert: TCheckBox;
    CheckExpand: TCheckBox;
    LBlock: TLabel;
    LBlock2: TLabel;
    EBLocking: TMaskEdit;
    IBBackup: TIBBackupService;
    procedure DMemtableNewRecord(DataSet: TDataSet);
    procedure RadioServiceClick(Sender: TObject);
    procedure DMemtableUnitValidate(Sender: TField);
    procedure BOKClick(Sender: TObject);
    procedure BShowopsClick(Sender: TObject);
    procedure BHideOpsClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    role:string;
  end;

procedure BackupIBdatabase(var username,password,role,server,database:string;var protocol:TRpIBXProtocol;dbconf:TRpdbibconf);

implementation

{$R *.xfm}

procedure BackupIBdatabase(var username,password,role,server,database:string;var protocol:TRpIBXProtocol;dbconf:TRpDbibconf);
var
 dia:TFBackup;
begin
 dia:=TFBackup.create(Application);
 Application.createform(TFBackup,dia);
  try
  dia.Eusername.text:=username;
  dia.EPassword.text:=password;
  dia.Erole.text:=role;
  dia.EServer.text:=server;
  dia.dbconf:=dbconf;
  dia.EServerChange(dia);
  dia.EBase.text:=Database;
  dia.comboprotocol.itemindex:=integer(protocol);
  dia.Radiocommandline.checked:=ibman.globalusecommandline;
  dia.showmodal;
  if dia.executed then
  begin
   username:=dia.USername;
   password:=dia.Password;
   role:=dia.role;
   database:=dia.database;
   server:=dia.Server;
   protocol:=dia.protocol;
  end;
 finally
  dia.free;
 end;
end;

procedure TFBackup.DMemtableNewRecord(DataSet: TDataSet);
begin
  inherited;
 DMemTableUnit.Value:='Megabytes';
end;

procedure TFBackup.RadioServiceClick(Sender: TObject);
begin
  inherited;
 CheckService.Enabled:=RadioCommandline.checked;
 CheckExpand.Enabled:=RadioCommandline.checked;
end;

procedure TFBackup.DMemtableUnitValidate(Sender: TField);
begin
  inherited;
  if TField(Sender).AsString<>'Kilobytes' then
   if TField(Sender).AsString<>'Megabytes' then
    if TField(Sender).AsString<>'Gigabytes' then
     Raise exception.Create(SRpInvalidFieldValue);
end;

procedure TFBackup.BOKClick(Sender: TObject);
var
 server:string;
 asize:integer;
 maxsize,thesize:int64;
begin
  inherited;
 role:=Erole.text;
 DMemTable.checkbrowsemode;
 maxsize:=MaxInt;
 if (DMemtable.EOF AND DMemTable.BOF) then
  Raise Exception.Create(SIBEspeFiles);
 server:='';
 DMemtable.disablecontrols;
 try
  // Check for correct values
  DMemTable.first;
  While Not DMemTable.eof do
  begin
   if DMemTableSize.IsNull then
    break;
   DMemTable.next;
  end;
  DMemtable.next;
  if Not DMemTable.EOF then
   Raise Exception.Create(SIBSecTamany);
  if Not DMemTablesize.isnull then
  begin
   DMemTable.edit;
   try
    DMemTablesize.asvariant:=null;
    DMemTable.post;
   except
    DMemTable.cancel;
    raisE;
   end;
  end;
  DMemTable.first;
  IBBackup.Backupfile.clear;
  IBBackup.ServerName:=Trim(EServer.Text);
  IBBackup.databasename:=EBase.text;
  ibman1.serverpath:=ibman.globalserverpath;
  Ibman1.filename:=BuildIBDatabaseName((Length(Trim(EServer.text))>0),TRpIBXProtocol(ComboProtocol.ItemIndex),EServer.Text,EBase.text);
  ibman1.backupfiles.clear;
  While Not DMemtable.eof do
  begin
   if Not DMemTableSize.isnull then
   begin
    ibman1.backupfiles.Add(DMemtableFile.asstring);
    ibman1.backupfiles.Add(DMemTableSize.asstring+Lowercase(DMemtableunit.AsString[1]));
    case  Lowercase(DMemtableunit.AsString[1])[1] of
     'k':
      begin
       thesize:=dmemtablesize.asinteger;
       thesize:=thesize*1024;
      end;
     'm':
      begin
       thesize:=dmemtablesize.asinteger;
       thesize:=thesize*1024*1024;
      end;
      else
      begin
       thesize:=dmemtablesize.asinteger;
       thesize:=thesize*1024*1024*1024;
      end;
    end;
    if thesize>maxsize then
     thesize:=maxsize;
    asize:=integer(thesize);
    IBBackup.BlockingFactor:=StrToInt(EBlocking.Text);
    IBBackup.Backupfile.add(dmemtablefile.asstring
     +'='+IntToStr(asize));
   end
   else
   begin
    IBBackup.Backupfile.add(dmemtablefile.asstring);
    ibman1.backupfiles.Add(DMemtableFile.asstring);
   end;
   DMemTable.next;
  end;
  if RadioService.checked then
  begin
   IBBackup.Params.clear;
   IBBackup.params.add('user_name='+EUsername.text);
   IBBackup.params.add('password='+EPassword.text);
   if Length(Trim(Erole.text))>0 then
    IBBackup.params.add('sql_role_name='+Erole.text);
   if Length(IBBackup.ServerName)=0 then
   begin
    IBBackup.protocol:=Local;
   end
   else
   begin
    IBBackup.protocol:=TProtocol(ComboProtocol.itemindex);
   end;
   IBBackup.options:=[];
   if Checkignore.checked then
    IBBackup.options:=IBBackup.options+[Ignorechecksums];
   if Checktrans.checked then
    IBBackup.options:=IBBackup.options+[Ignorelimbo];
   if CheckMeta.checked then
    IBBackup.options:=IBBackup.options+[Metadataonly];
   if CheckNoGarbage.checked then
    IBBackup.options:=IBBackup.options+[NoGarbagecollection];
   if CheckOldMeta.checked then
    IBBackup.options:=IBBackup.options+[OldMetadatadesc];
   if CheckNotrans.checked then
    IBBackup.options:=IBBackup.options+[NonTransportable];
   if CheckConvert.checked then
    IBBackup.options:=IBBackup.options+[ConvertExtTables];

   IBBackup.Attach;
   try
    IBBackup.ServiceStart;
    ibproces.COntrolIBService(IBBackup);
   finally
    try
     IBBackup.Detach;
    except
    end;
   end;
  end
  else
  begin
   ibman1.username:=EUsername.text;
   ibman1.Password:=EPassword.text;
   ibman1.BackupOptions:=[];
   if CheckIgnore.checked then
    ibman1.BackupOptions:=ibman1.backupOptions+[bIgnoreChecksums];
   if CheckConvert.checked then
    ibman1.BackupOptions:=ibman1.backupOptions+[bConverttables];
   if CheckTrans.checked then
    ibman1.BackupOptions:=ibman1.backupOptions+[bIgnorelimbo];
   if CheckMeta.checked then
    ibman1.BackupOptions:=ibman1.backupOptions+[bMetadataonly];
   if CheckNoGarbage.checked then
    ibman1.BackupOptions:=ibman1.backupOptions+[bNogarbagecollection];
   if CheckNoTrans.checked then
    ibman1.BackupOptions:=ibman1.backupOptions+[bNonTransportable];
   if CheckOldMeta.checked then
    ibman1.BackupOptions:=ibman1.backupOptions+[bOldMetadata];
   if CheckExpand.checked then
    ibman1.BackupOptions:=ibman1.backupOptions+[bExpand];
   ibman1.BackBlockingfactor:=StrToInt(EBlocking.Text);
   if Checkservice.checked then
   begin
    if length(Trim(Eserver.Text))>0 then
    begin
     case Comboprotocol.itemindex of
      0:
       begin
        ibman1.service:=Trim(EServer.text)+':'+'service_mgr';
       end;
      1:
       begin
        ibman1.service:='\\'+Trim(EServer.text)+'\'+'service_mgr';
       end;
      else
      begin
       ibman1.service:=Trim(EServer.text)+'@'+'service_mgr';
      end;
     end;
    end
    else
    begin
     ibman1.service:='service_mgr';
    end;
   end;
   ibman1.Backup;
  end;
 finally
  DMemTable.enablecontrols;
 end;
end;


procedure TFBackup.BShowopsClick(Sender: TObject);
begin
 Width:=GOptions.Left+Goptions.Width+5;
 BhideOps.Visible:=True;
 BShowOps.Visible:=False;
 GOptions.Visible:=True;
end;

procedure TFBackup.BHideOpsClick(Sender: TObject);
begin
 Width:=BShowops.Left+BShowOps.Width+5;
 BhideOps.Visible:=False;
 BShowOps.Visible:=True;
 GOptions.Visible:=False;
end;

procedure TFBackup.FormActivate(Sender: TObject);
begin
  inherited;
 DMemtable.close;
 DMemtable.createdataset;
 Width:=BShowops.Left+BShowOps.Width+5;
end;

end.
