{*******************************************************}
{                                                       }
{       IBAccess                                        }
{       New index wizard                                }
{                                                       }
{       Author:                                         }
{       Toni Martir                                     }
{       http://www.ibaccess.org                         }
{       e-mail: toni@pala.com                           }
{                                                       }
{       OpenSource license (MPL)                        }
{                                                       }
{*******************************************************}

unit ibidxnew;

interface

uses Classes, SysUtils,QForms, QControls, QStdCtrls,
  QButtons, QExtCtrls,ibdatabase, IBSQL, Db,
  ibacutils,ibacconst;

type
  TFIndexNew = class(TForm)
    OKBtn: TButton;
    CancelBtn: TButton;
    Label1: TLabel;
    LTablename: TLabel;
    Trans: TIBTransaction;
    IBSQL1: TIBSQL;
    Label2: TLabel;
    EIndexName: TEdit;
    Label3: TLabel;
    LAval: TListBox;
    BADD: TBitBtn;
    BRemove: TBitBtn;
    LSelected: TListBox;
    CheckDescending: TCheckBox;
    CheckUnique: TCheckBox;
    Label4: TLabel;
    Bup: TBitBtn;
    BDown: TBitBtn;
    procedure OKBtnClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure BADDClick(Sender: TObject);
    procedure BRemoveClick(Sender: TObject);
    procedure BupClick(Sender: TObject);
    procedure BDownClick(Sender: TObject);
  private
    { Private declarations }
    fieldlist:TStringList;
    tablename:string;
    procedure DoCreateIndex;
    procedure fillavalfields(dia:TFFetching);
  public
    { Public declarations }
    base:TIBDatabase;
    created:boolean;
  end;

function ibnewindex(base:TIBDatabase;table:string):boolean;

implementation


{$R *.xfm}

function ibnewindex(base:TIBDatabase;table:string):boolean;
var
 dia:TFIndexNew;
begin
 dia:=TFIndexNew.create(Application);
 try
  dia.trans.defaultdatabase:=base;
  dia.ibsql1.database:=base;
  dia.tablename:=table;
  dia.base:=base;
  dia.showmodal;
  Result:=dia.created;
 finally
  dia.free;
 end;
end;

procedure TFIndexNew.OKBtnClick(Sender: TObject);
begin
 DoCreateIndex;
 created:=true;
 Close;
end;


procedure TFIndexNew.DoCreateIndex;
var
 i:integer;
 adeli:string;
begin
 adeli:='';
 if base.sqldialect=3 then
  adeli:='"';
 // Creates the index
 if LSelected.items.count<1 then
  Raise Exception.Create(SRpNoFieldsSelected);
 Trans.starttransaction;
 try
  IBSQL1.sql.text:='CREATE ';
  if CheckUnique.checked then
   IBSQL1.sql.text:=IBSQL1.sql.text+'UNIQUE ';
  if CheckDescending.checked then
   IBSQL1.sql.text:=IBSQL1.sql.text+' DESCENDING ';
  IBSQL1.sql.text:=IBSQL1.sql.text+' INDEX  '+
   EIndexname.text+' ON '+adeli+Tablename+adeli+' (';
  IBSQL1.sql.text:=IBSQL1.sql.text+adeli+lselected.items.strings[0]+adeli;
  for i:=1 to lselected.items.count-1 do
  begin
   IBSQL1.sql.text:=IBSQL1.sql.text+','+adeli+lselected.items.strings[i]+adeli;
  end;
  IBSQL1.sql.text:=IBSQL1.sql.text+')';
  IBSQL1.ExecQuery;
  trans.commit;
 except
  trans.rollback;
  raise;
 end;
end;

procedure TFIndexNew.fillavalfields(dia:TFFetching);
begin
 Trans.starttransaction;
 try
  IBSQL1.sql.text:='SELECT RDB$FIELD_NAME FROM RDB$RELATION_FIELDS '+
   ' WHERE RDB$RELATION_NAME='''+tablename+''' ORDER BY RDB$FIELD_POSITION';
  IBSQL1.ExecQuery;
  try
   While Not IBSQL1.EOF do
   begin
    Fieldlist.add(IBSQL1.FieldByName('RDB$FIELD_NAME').AsString);
    IBSQL1.Next;
    dia.IncFetch(1);
   end;
  finally
   IBSQL1.CLose;
  end;
 finally
  trans.rollback;
 end;
 LAval.items.assign(Fieldlist);
 if fieldlist.count>0 then
  LAval.itemindex:=0;
end;

procedure TFIndexNew.FormShow(Sender: TObject);
begin
 LTablename.caption:=tablename;
 EIndexName.text:='I'+tablename+'NEWINDEX';
 ExecuteActionProgres(fillavalfields,true,nil);
end;

procedure TFIndexNew.FormCreate(Sender: TObject);
begin
 fieldlist:=TStringlist.create;
// SendMessage(LAval.Handle,LB_SETHORIZONTALEXTENT,512,0);
// SendMessage(LSelected.Handle,LB_SETHORIZONTALEXTENT,512,0);
end;

procedure TFIndexNew.FormDestroy(Sender: TObject);
begin
 fieldlist.free;
end;

procedure TFIndexNew.BADDClick(Sender: TObject);
begin
 if LAval.items.count<1 then
  exit;
 if lAval.itemindex<0 then
  exit;
 LSelected.Items.Add(laval.items.strings[laval.itemindex]);
 LAval.items.delete(laval.itemindex);
 if LSelected.items.count=1 then
  LSelected.itemindex:=0;
 if LAval.items.count>0 then
  LAval.itemindex:=0;
end;

procedure TFIndexNew.BRemoveClick(Sender: TObject);
var
 oldaval:string;
 alist:TStringlist;
 i,index:integer;
begin
 if LSelected.items.count<1 then
  exit;
 if lSelected.itemindex<0 then
  exit;
 index:=lselected.itemindex;
 LSelected.items.delete(lselected.itemindex);
 if LSelected.items.count>0 then
  LSelected.itemindex:=index;
 // Restores all the fields to position the field
 alist:=TStringlist.create;
 try
  oldaval:='';
  if LAval.itemindex>=0 then
   oldaval:=LAval.items.strings[laval.itemindex];
  alist.assign(fieldlist);
  for i:=0 to LSelected.items.count-1 do
  begin
   index:=alist.indexof(LSelected.items.strings[i]);
   if index>=0 then
    alist.delete(index);
  end;
  LAval.items.assign(alist);
  index:=laval.items.indexof(oldaval);
  if index>=0 then
   Laval.itemindex:=index;
 finally
  alist.free;
 end;
end;

procedure TFIndexNew.BupClick(Sender: TObject);
var
 afield:string;
begin
 if LSelected.items.count<1 then
  exit;
 if lSelected.itemindex<1 then
  exit;
 with lselected do
 begin
  afield:=Items.strings[itemindex-1];
  items.strings[itemindex-1]:=items.strings[itemindex];
  items.strings[itemindex]:=afield;
  itemindex:=itemindex-1;
 end;
end;

procedure TFIndexNew.BDownClick(Sender: TObject);
var
 afield:string;
begin
 if LSelected.items.count<1 then
  exit;
 if lSelected.itemindex<0 then
  exit;
 if lSelected.itemindex>=lselected.items.count-1 then
  exit;
 with lselected do
 begin
  afield:=Items.strings[itemindex+1];
  items.strings[itemindex+1]:=items.strings[itemindex];
  items.strings[itemindex]:=afield;
  itemindex:=itemindex+1;
 end;
end;

end.
