{*******************************************************}
{                                                       }
{       IBAccess                                        }
{       New domain and alter domain dialog              }
{                                                       }
{       Author:                                         }
{       Toni Martir                                     }
{       http://www.ibaccess.org                         }
{       e-mail: toni@pala.com                           }
{                                                       }
{       OpenSource license (MPL)                        }
{                                                       }
{*******************************************************}

unit ibnewdom;

interface

uses
  SysUtils, Classes, QForms,variants,
  QMask, ibdatabase, IBSQL, ibacconst,
  Db, DBClient, QStdCtrls, QControls, QGrids, QDBGrids;

type
  TFIBNewDom = class(TForm)
    Button1: TButton;
    Button2: TButton;
    IBCustom: TIBSQL;
    tran1: TIBTransaction;
    DDimensions: TClientdataset;
    DDimensionsLowIndex: TIntegerField;
    DDimensionsHighIndex: TIntegerField;
    DataSource1: TDataSource;
    LArray: TLabel;
    GridArray: TDBGrid;
    ECheck: TEdit;
    Label9: TLabel;
    Label7: TLabel;
    EDefault: TEdit;
    Label8: TLabel;
    Label10: TLabel;
    GOrdena: TGroupBox;
    Label11: TLabel;
    Lcoll: TLabel;
    ComboChar: TComboBox;
    ComboColl: TComboBox;
    GChar: TGroupBox;
    Longitud: TLabel;
    Label15: TLabel;
    ELongchar: TMaskEdit;
    ComboType: TComboBox;
    Label2: TLabel;
    Label1: TLabel;
    EDomain: TEdit;
    CheckNoNull: TCheckBox;
    GNum: TGroupBox;
    Label14: TLabel;
    Label16: TLabel;
    Label17: TLabel;
    wrweqr: TLabel;
    EDecimals: TMaskEdit;
    Eprecision: TMaskEdit;
    GBlob: TGroupBox;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    ESegLen: TMaskEdit;
    ESubtype: TMaskEdit;
    procedure Button1Click(Sender: TObject);
    procedure ComboTypeClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ComboCollDropDown(Sender: TObject);
    procedure ESubtypeChange(Sender: TObject);
  private
    { Private declarations }
    formodify:boolean;
    domain:string;
    oldtype,oldcheck,olddefault:string;
    function Obtainstringtype(onlytype:boolean):string;
    procedure Createdomain;
    procedure Fillinfo;
    procedure Modifydomain;
  public
    { Public declarations }
    base:TIBDatabase;
    docreate:boolean;
    function buildsource:string;
  end;


function ibmodifydomain(base:TIBDatabase;domain:string):boolean;
function ibnewdomain(base:TIBDatabase;domain:string):boolean;
procedure IBDropdomain(base:TIBDatabase;domain:string);

implementation

uses ibutil2;

{$R *.xfm}

function ibmodifydomain(base:TIBDatabase;domain:string):boolean;
var
 dia:TFIBNewDom;
begin
 dia:=TFIBNewdom.create(Application);
 try
  dia.tran1.defaultdatabase:=base;
  dia.IBCustom.database:=base;
  dia.base:=base;
  dia.domain:=domain;
  dia.HelpContext:=250;
  dia.gridarray.readonly:=True;
//  dia.gridarray.color:=clinfobk;
  if dia.base.sqldialect=3 then
  begin
   dia.ComboType.Items.Insert(6,'TIME');
   dia.ComboType.Items.Insert(7,'TIMESTAMP');
  end;
  dia.ComboTypeclick(dia);
  dia.Fillinfo;
  dia.showmodal;
  Result:=dia.docreate;
 finally
  dia.free;
 end;
end;

function ibnewdomain(base:TIBDatabase;domain:string):boolean;
var
 dia:TFIBNewDom;
begin
 dia:=TFIBNewDom.create(Application);
 try
  dia.tran1.defaultdatabase:=base;
  dia.IBCustom.database:=base;
  dia.domain:=domain;
  dia.base:=base;
  if dia.base.sqldialect=3 then
  begin
   dia.ComboType.Items.Insert(6,'TIME');
   dia.ComboType.Items.Insert(7,'TIMESTAMP');
  end;
  dia.ComboTypeclick(dia);
  dia.showmodal;
  Result:=dia.docreate;
 finally
  dia.free;
 end;
end;

procedure TFIBNewDom.Button1Click(Sender: TObject);
begin
 if not formodify then
  Createdomain
 else
  Modifydomain;
 docreate:=True;
 Close;
end;

procedure TFIBNewDom.Createdomain;
begin
 IBExecuteSQL(base,buildsource);
end;

procedure TFIBNewDom.ComboTypeClick(Sender: TObject);
begin
 GBlob.visible:=false;
 GOrdena.visible:=false;
 GNum.visible:=false;
 GChar.visible:=false;
 LArray.visible:=true;
 GridArray.visible:=true;
 ComboChar.itemindex:=-1;
 ComboColl.itemindex:=-1;
 if base.sqldialect=1 then
 begin
  case ComboType.itemindex of
   6,7:
    begin
     GChar.visible:=true;
     GOrdena.visible:=true;
     LColl.Visible:=true;
     ComboColl.visible:=true;
     if (length(base.Params.Values['lc_ctype'])>0) then
      ComboChar.itemindex:=Combochar.Items.IndexOf(base.Params.Values['lc_ctype'])
     else
      ComboChar.itemindex:=0;
    end;
   8:
    begin
     GBlob.visible:=true;
     GOrdena.visible:=true;
     if (length(base.Params.Values['lc_ctype'])>0) then
      ComboChar.itemindex:=Combochar.Items.IndexOf(base.Params.Values['lc_ctype'])
     else
      ComboChar.itemindex:=0;
     LColl.Visible:=false;
     ComboColl.visible:=false;
     LArray.visible:=false;
     GridArray.visible:=false;
     DDimensions.Close;
     DDimensions.Createdataset;
    end;
   4:
    begin
     GNum.visible:=true;
    end;
  end;
 end
 else
 begin
  case ComboType.itemindex of
   8,9:
    begin
     GChar.visible:=true;
     GOrdena.visible:=true;
     LColl.Visible:=true;
     ComboColl.visible:=true;
     if (length(base.Params.Values['lc_ctype'])>0) then
      ComboChar.itemindex:=Combochar.Items.IndexOf(base.Params.Values['lc_ctype'])
     else
      ComboChar.itemindex:=0;
    end;
   10:
    begin
     GBlob.visible:=true;
     if (length(base.Params.Values['lc_ctype'])>0) then
      ComboChar.itemindex:=Combochar.Items.IndexOf(base.Params.Values['lc_ctype'])
     else
      ComboChar.itemindex:=0;
     try
      Gordena.Visible:=(StrToInt(ESubtype.TExt)=1);
     except
      GOrdena.Visible:=False;
     end;
     LColl.Visible:=false;
     ComboColl.visible:=false;
     LArray.visible:=false;
     GridArray.visible:=false;
     DDimensions.Close;
     DDimensions.Createdataset;
    end;
   4:
    begin
     GNum.visible:=true;
    end;
  end;
 end;
end;

procedure TFIBNewDom.FormCreate(Sender: TObject);
begin
 DDimensions.createdataset;
 ComboType.itemindex:=1;
end;



procedure IBDropdomain(base:TIBDatabase;domain:string);
begin
 if base.sqldialect=3 then
  IBExecuteSQL(base,'DROP DOMAIN "'+domain+'"')
 else
  IBExecuteSQL(base,'DROP DOMAIN '+domain)
end;

function TFIBNewDom.buildsource:string;
var
 adeli:String;
begin
 DDimensions.CheckBrowseMode;
 adeli:='';
 if base.sqldialect=3 then
  adeli:='"';
 Result:='CREATE DOMAIN '+adeli+Trim(EDomain.text)+adeli+' AS '+#13+#10;
 Result:=Result+ObtainStringType(false);
 Result:=Result+#13+#10;

 if Length(Trim(EDefault.text))>0 then
  Result:=Result+EDefault.text+#13+#10;
 if CheckNoNull.checked then
  Result:=Result+' NOT NULL'+#13+#10;
 if Length(Trim(ECheck.text))>0 then
  Result:=Result+ECheck.text+#13+#10;
 if ( ((base.sqldialect=1) and (ComboType.ItemIndex in [6,7]))
      OR ((base.sqldialect<>1) and (ComboType.ItemIndex in [8,9])) )then
 begin
  if Length(Trim(COmboColl.text))>0 then
   Result:=Result+' COLLATE '+ComboColl.text;
 end;
end;

procedure TFIBNewDom.Fillinfo;
var
 dinfo:TIBDefdomain;
 i:integer;
begin
 formodify:=True;
 // Search and assign new domain properties
 Caption:=SRpModifydomain;
 Checknonull.enabled:=false;
 dinfo:=Filldomaininfo(base,domain,nil);
 oldtype:=ibfieldtypetosource(dinfo.basetype,dinfo.size,dinfo.scale,base.sqldialect);
 if oldtype='BLOB' then
  oldtype:=oldtype+' SUB_TYPE '+inttostr(dinfo.subtype)+' SEGMENT SIZE '+inttostr(dinfo.segmentsize);
 if Length(dinfo.character_set)>0 then
  oldtype:=oldtype+' CHARACTER SET '+dinfo.character_set;
 // ibfieldtypetosourcescale(dinfo.basetype,dinfo.size,dinfo.scale,dinfo.precision,dinfo.precscale,base.sqldialect);
 oldcheck:=dinfo.check;
 olddefault:=dinfo.default;
 EDomain.text:=domain;
 ECheck.Text:=dinfo.check;
 EDefault.text:=dinfo.default;
 ComboType.itemindex:=ComboType.items.IndexOf(ibfieldtypetostringtype(dinfo.basetype,dinfo.scale,base.sqldialect));
 ComboTypeClick(Self);
 if length(dinfo.character_set)>0 then
 begin
  ComboChar.itemindex:=Combochar.Items.IndexOf(dinfo.character_set);
  Combocoll.text:=dinfo.collation;
 end;
 DDimensions.disablecontrols;
 try
  for i:=0 to dinfo.arraydimcount-1 do
  begin
   DDimensions.Append;
   try
    DDimensionsLowIndex.Value:=dinfo.arraydim[i].low;
    DDimensionsHighIndex.Value:=dinfo.arraydim[i].high;
    DDimensions.post;
   except
    DDimensions.cancel;
    raise;
   end;
  end;
 finally
  DDimensions.EnableControls;
 end;
 if dinfo.subtype<>null then
  ESubtype.text:=Inttostr(dinfo.subtype);
 if dinfo.segmentsize<>null then
  ESegLen.text:=Inttostr(dinfo.segmentsize);
 if dinfo.precision<>null then
  EPrecision.text:=Inttostr(dinfo.precision);
 if dinfo.scale<>null then
  EDecimals.text:=Inttostr(-dinfo.scale);
 ELongChar.text:=Inttostr(dinfo.size);
end;

procedure TFIBNewDom.Modifydomain;
var
 newname:string;
 newtype:string;
 newcheck:string;
 newdefault:string;
 adeli:string;
begin
 // Executem
 adeli:='';
 if base.sqldialect=3 then
  adeli:='"';
 newname:=Trim(Edomain.Text);
 if Length(newname)<0 then
 begin
  Raise Exception.create(SRpDomainName);
 end;
 tran1.StartTransaction;
 try
  newcheck:=Trim(ECheck.Text);
  if newcheck<>oldcheck then
  begin
   IBCustom.SQL.Text:='ALTER DOMAIN '+adeli+domain+adeli+' DROP CONSTRAINT';
   IBCustom.ExecQuery;
   IBCustom.SQL.Text:='ALTER DOMAIN '+adeli+domain+adeli+' ADD  '+ECheck.text;
   IBCustom.ExecQuery;
  end;
  newdefault:=Trim(EDefault.Text);
  if newdefault<>olddefault then
  begin
   IBCustom.SQL.Text:='ALTER DOMAIN '+adeli+domain+adeli+' DROP DEFAULT';
   IBCustom.ExecQuery;
   IBCustom.SQL.Text:='ALTER DOMAIN '+adeli+domain+adeli+' SET  '+EDefault.text;
   IBCustom.ExecQuery;
  end;
  if newname<>domain then
  begin
   IBCustom.SQL.TExt:='ALTER DOMAIN '+adeli+domain+adeli+' TO '+adeli+newname+adeli;
   IBCustom.ExecQuery;
  end;
  newtype:=Obtainstringtype(true);
  if newtype<>oldtype then
  begin
   IBCustom.SQL.TExt:='ALTER DOMAIN '+adeli+domain+adeli+' TYPE '+newtype;
   IBCustom.ExecQuery;
  end;
  tran1.commit;
 except
  tran1.rollback;
  raise;
 end;
end;

function TFIBNewDom.Obtainstringtype(onlytype:boolean):string;
var
 astring:string;
begin
 Result:=ComboType.items.strings[ComboType.itemindex];
 if base.sqldialect=1 then
 begin
  case ComboType.itemindex of
   6,7:
    Result:=Result+'('+IntToStr(StrToInt(ELongchar.Text))+')';
   4:
    Result:=Result+'('+IntToStr(StrToInt(Eprecision.Text))+','+inttostr(StrToInt(EDecimals.Text))+')';
  end;
 end
 else
 begin
  case ComboType.itemindex of
   8,9:
    Result:=Result+'('+IntToStr(StrToInt(ELongchar.Text))+')';
   4:
    Result:=Result+'('+IntToStr(StrToInt(Eprecision.Text))+','+inttostr(StrToInt(EDecimals.Text))+')';
  end;
 end;
 if Not onlytype then
 begin
  // Add the dimensions
  DDimensions.first;
  if Not DDimensions.eof then
  begin
   astring:='[';
   While Not DDimensions.EOF do
   begin
    astring:=astring+DDimensionsLowIndex.AsString+':'+
     DDimensionsHighIndex.AsString;
    DDimensions.next;
    if Not DDimensions.EOF then
     astring:=astring+',';
   end;
   astring:=astring+']';
  end;
  Result:=Result+astring;
 end;
 if ( ((base.sqldialect=1) and (ComboType.ItemIndex in [6,7]))
      OR ((base.sqldialect<>1) and (ComboType.ItemIndex in [8,9])) )then
 begin
  Result:=Result+' CHARACTER SET '+ComboChar.text;
 end;
 if  (  ((base.sqldialect=1) AND (ComboType.ItemIndex=8)) OR
     ((base.sqldialect<>1) AND (ComboType.ItemIndex=10)) )
    then
 begin
  Result:=Result+' SUB_TYPE '+IntTostr(StrToInt(ESubTYpe.TExt))+' SEGMENT SIZE '+IntTostr(StrToInt(ESegLen.TExt));
  if (StrToInt(ESubTYpe.TExt)=1) then
   Result:=Result+' CHARACTER SET '+ComboChar.text;
 end;
{ if onlytype then
 begin
  if ( ((base.sqldialect=1) and (ComboType.ItemIndex in [6,7]))
       OR ((base.sqldialect<>1) and (ComboType.ItemIndex in [8,9])) )then
  begin
   Result:=Result+' COLLATE '+ComboColl.text;
  end;
 end;
}end;

procedure TFIBNewDom.ComboCollDropDown(Sender: TObject);
var
 orderdef:string;
 charid:integer;
begin
 // Posibles collation orders
 ComboColl.items.clear;
 if ComboChar.Itemindex<0 then
  exit;
 if Tran1.InTransaction then
  Tran1.Rollback;
 tran1.StartTransaction;
 try
  IBCustom.Close;
  IBCustom.SQL.TExt:='SELECT RDB$CHARACTER_SET_ID CHARID,RDB$DEFAULT_COLLATE_NAME DEFCOL'+
   ' FROM RDB$CHARACTER_SETS WHERE RDB$CHARACTER_SET_NAME='''+
   combochar.items.strings[combochar.itemindex]+'''';
  IBCustom.ExecQuery;
  try
   charid:=1;
   If Not IBCustom.EOF then
   begin
    charid:=IBCustom.fieldbyname('CHARID').AsInteger;
    orderdef:=Trim(IBCustom.fieldbyname('DEFCOL').AsString);
   end
   else
    exit;
   IBCustom.close;
   IBCustom.sql.text:='SELECT RDB$COLLATION_NAME NOM '+
   ' FROM RDB$COLLATIONS WHERE RDB$CHARACTER_SET_ID='+
    inttostr(charid);
   IBCustom.execquery;
   While not IBCustom.eof do
   begin
    ComboColl.Items.Add(Trim(IBCustom.fieldbyname('NOM').AsString));
    IBCustom.next;
   end;
  finally
   IBCustom.close;
  end;
  tran1.commit;
 except
  tran1.rollback;
  raise;
 end;
end;

procedure TFIBNewDom.ESubtypeChange(Sender: TObject);
begin
 try
  GOrdena.Visible:=(StrToInt(ESubtype.Text)=1);
 except
  GOrdena.Visible:=False;
 end;
end;

end.
