{*******************************************************}
{                                                       }
{       Rpibdb                                          }
{       TRpDbibconf: A component that stores            }
{       database connection information of a            }
{       collection of ibdatabases                       }
{       The info is stored on a user roaming ini file   }
{       or on a system inifile                          }
{                                                       }
{       Author:                                         }
{       Toni Martir                                     }
{       http://www.ibaccess.org                         }
{       e-mail: toni@pala.com                           }
{                                                       }
{       OpenSource license (MPL)                        }
{                                                       }
{*******************************************************}


unit rpibdb;

interface

uses
  SysUtils, Classes,QGraphics, QControls, QForms, QDialogs,
  IBDatabase, Db, inifiles,
  rpdconsts, rpstdutils,
  typinfo,rpshfolder, QButtons, QStdCtrls,QExtCtrls;

type
  TRpDBIBConf=class;

  TRpIBXProtocol=(rpIBXTCP,rpIBXNetBeui,rpIBXIpx);


  TRpDbIBInfo=class(TCollectionItem)
   private
    FConnectionType:integer;
    // Base for all databases
    FLoginPrompt:boolean;
    FConnected:boolean;
    FUsername,FPassword:string;
    FConnectionAlias:string;
    // IBX
    FServer:string;
    FFilename:string;
    Fcharacterset:string;
    FRole:string;
    FRemote:boolean;
    FProtocol:TRpIBXProtocol;
    FSQLDialect:integer;
    // All
    FDatabase:TIBDatabase;
    // SetFunctions for all databases
    procedure SetUserName(Value:string);
    procedure SetPassword(Value:string);
    procedure SetConnectionAlias(Value:string);
    procedure SetConnected(Value:boolean);
    procedure SetLoginPrompt(Value:boolean);
    // SetFunctions IBX
    procedure SetServer(Value:string);
    procedure SetFilename(Value:string);
    procedure SetCharacterset(Value:string);
    procedure SetRole(Value:string);
    procedure SetRemote(Value:boolean);
    procedure SetProtocol(Value:TRpIBXProtocol);
    procedure SetSQLDialect(Value:integer);
    procedure ConnectIBX(dbase:TIBDatabase);
    function GetIBXFilename:string;
    procedure SetDatabase(Value:TIBDatabase);
   public
    constructor Create(Collection:TCollection);override;
    procedure Assign(Source:TPersistent);override;
    procedure GetTableNames(TableNames:TStrings);
    procedure DisconnectDatabase;
    function ConnectDatabase:Boolean;
    procedure LoadConfig(Index:integer;inifile:TInifile);
    procedure SaveConfig(Index:integer;inifile:TInifile);
    property  IBXFilename:string read GetIBXFilename;
    procedure CreateDatabase(AOwner:TCOmponent);
   protected
   published
    // All databases properties
    property UserName:string read FUserName write SetUserName;
    property Password:string read FPassword write SetPassword;
    property Connected:boolean read FConnected write SetConnected default True;
    property LoginPrompt:boolean read FLoginPrompt write SetLoginPrompt default False;
    property ConnectionAlias:string read FConnectionAlias write SetConnectionAlias;
    // IBX properties
    property Server:string read FServer write SetServer;
    property Filename:string read FFilename write SetFilename;
    property CharacterSet:string read FCharacterSet write SetCharacterSet;
    property Role:string read FRole write SetRole;
    property Remote:boolean read FRemote write SetRemote default false;
    property Protocol:TRpIBXProtocol read FProtocol write SetProtocol default rpIBXTCP;
    property SQLDIalect:integer read FSQLDialect write SetSQLDialect default 1;
    property Database:TIBDatabase read FDatabase write SetDatabase;
  end;

  TRpDBIBList=class(TCollection)
   private
    FDatabaseConf:TRpDBIBConf;
    function GetItem(Index:Integer):TRpDbIBInfo;
    procedure SetItem(index:integer;Value:TRpDbIBInfo);
   protected
   public
    Constructor Create(dbconf:TRpDBIBConf);
    function Add(ConnectionAlias:string):TRpDbIBInfo;
    function IndexOf(ConnectionAlias:string):integer;
    procedure AssignTo(Dest:TPersistent);override;
    property Items[index:integer]:TRpDbIBInfo read GetItem write SetItem;default;
   end;

  TRpDBIBConf=class(TComponent)
  private
   FDatabases:TRpDBIBList;
   FConnected:Boolean;
   FConfigurationIndex:integer;
   FSystemConfig:boolean;
   FAliasConfig:boolean;
   FHelpcontext:integer;
   procedure SetDatabases(Value:TRpDBIBList);
   procedure SetConnected(Value:Boolean);
  protected
   procedure Notification(AComponent:TComponent;Operation:TOperation);override;
  public
   constructor Create(AOwner:TComponent);override;
   destructor Destroy;override;
   procedure Loaded;override;
   procedure ShowConfig(AllowLoadandsave,AllowNewDelete,autosave:boolean);overload;
   procedure ShowConfig(AOwner:TCOmponent);overload;
   procedure LoadConfig;
   procedure SaveConfig;
   procedure LoadAliases;
   procedure SaveAliases;
  published
   property Databases:TRpDBIBList read FDatabases write SetDatabases;
   property Connected:Boolean read FConnected write SetConnected default false;
   property ConfigurationIndex:integer read FConfigurationIndex write FConfigurationIndex default 0;
   property Systemconfig:boolean read FSystemconfig write FSystemconfig default True;
   property Aliasconfig:boolean read FAliasconfig write FAliasConfig default false;
   property Helpcontext:integer read FHelpContext write FHelpcontext default 0;
  end;



  TFRpIBDatabases = class(TForm)
    Panel1: TPanel;
    Splitter1: TSplitter;
    Pdefs: TPanel;
    PConect: TPanel;
    Label2: TLabel;
    Label3: TLabel;
    EUsername: TEdit;
    EPassword: TEdit;
    CheckLoginPrompt: TCheckBox;
    Panel3: TPanel;
    BOK: TButton;
    BCancel: TButton;
    BNew: TBitBtn;
    BRemove: TBitBtn;
    BLoad: TBitBtn;
    BSave: TBitBtn;
    BRename: TButton;
    BUp: TBitBtn;
    BDown: TBitBtn;
    BTest: TBitBtn;
    LDatabase: TLabel;
    ComboDatabase: TComboBox;
    DBList: TListBox;
    RadioLocal: TRadioButton;
    RadioRemote: TRadioButton;
    EServer: TEdit;
    LServer: TLabel;
    Label7: TLabel;
    EFilename: TEdit;
    LProtocol: TLabel;
    ComboProtocol: TComboBox;
    Erole: TEdit;
    Label10: TLabel;
    ComboCharset: TComboBox;
    Label9: TLabel;
    Label1: TLabel;
    ComboVersion: TComboBox;
    CheckConnected: TCheckBox;
    Label4: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure UpdateAlias;
    procedure RadioLocalClick(Sender: TObject);
    procedure EServerChange(Sender: TObject);
    procedure ComboProtocolChange(Sender: TObject);
    procedure EFilenameChange(Sender: TObject);
    procedure ComboCharsetChange(Sender: TObject);
    procedure EroleChange(Sender: TObject);
    procedure EUsernameChange(Sender: TObject);
    procedure EPasswordChange(Sender: TObject);
    procedure CheckLoginPromptClick(Sender: TObject);
    procedure CheckConnectedClick(Sender: TObject);
    procedure UpdateComboDatabase;
    procedure ComboDatabaseClick(Sender: TObject);
    procedure BNewClick(Sender: TObject);
    procedure DBListClick(Sender: TObject);
    procedure BRemoveClick(Sender: TObject);
    procedure BRenameClick(Sender: TObject);
    procedure BUpClick(Sender: TObject);
    procedure BDownClick(Sender: TObject);
    procedure ComboVersionClick(Sender: TObject);
    procedure BTestClick(Sender: TObject);
    procedure BLoadClick(Sender: TObject);
    procedure BSaveClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    { Private declarations }
    comboconnectionvisible:boolean;
    langnames,langdesc,langchar:TStringList;
{$IFDEF MSWINDOWS}
    designer:IFormDesigner;
{$ENDIF}
    databaseconf:TRpDBIBConf;
    aowner:TComponent;
    achange:boolean;
    loading:boolean;
    allowloadandsave,allownewdelete:boolean;
    function FindDatabaseinfo:TRpDbIBInfo;
    procedure ConnectionTypeChanged;
    procedure LocalRemoteChanged;
    procedure Addname(const aname:string);
    procedure Reloadlist;
  public
    { Public declarations }
  end;


{$IFDEF MSWINDOWS}
  // Propertty editor for TRpDBIBList
  TRpDBIBListPropEditor=class(TPropertyEditor)
  protected
  public
   function GetAttributes:TPropertyAttributes;override;
   procedure Edit;override;
   function GetValue:string;override;
  end;
{$ENDIF}

 function BuildIBDatabaseName(Remote:Boolean;Protocol:TRpIBXProtocol;Servername,Filename:string):string;

implementation

{$R *.xfm}

procedure TRpDbIBInfo.CreateDatabase(AOwner:TCOmponent);
begin
 if assigned(fdatabase) then
 begin
  if fdatabase.owner<>AOwner then
  begin
   fdatabase.free;
   fdatabase:=nil;
  end;
 end;
 if Assigned(database) then
  exit;
 fdatabase:=TIBDatabase.create(AOwner);
 generatenewname(database);
end;


// TRpDbIBInfo

constructor TRpDbIBInfo.Create(Collection:TCollection);
begin
 inherited Create(Collection);
 FConnectionType:=1;
 FCharacterset:='WIN1252';
 FConnected:=True;
 FSQLDialect:=1;
end;


procedure TRpDbIBInfo.Assign(Source:TPersistent);
begin
 if Source is TRpDbIBInfo then
 begin
  FDatabase:=TRpDbIBInfo(Source).FDatabase;
  FLoginPrompt:=TRpDbIBInfo(Source).FLoginPrompt;
  FConnected:=TRpDbIBInfo(Source).FConnected;
  FUsername:=TRpDbIBInfo(Source).Fusername;
  FPassword:=TRpDbIBInfo(Source).FPassword;
  FConnectionAlias:=TRpDbIBInfo(Source).FConnectionAlias;
  // IBX
  FServer:=TRpDbIBInfo(Source).FServer;
  FFileName:=TRpDbIBInfo(Source).FFileName;
  FCharacterSet:=TRpDbIBInfo(Source).FCharacterSet;
  FProtocol:=TRpDbIBInfo(Source).FProtocol;
  FRemote:=TRpDbIBInfo(Source).FRemote;
  FRole:=TRpDbIBInfo(Source).FRole;
  FSQLDialect:=TRpDbIBInfo(Source).FSQLDialect;
  Changed(False);
 end
 else
  inherited Assign(Source);
end;

procedure TRpDbIBInfo.SetUserName(Value:string);
begin
 Fusername:=Value;
 changed(False);
end;

procedure TRpDbIBInfo.SetPassword(Value:string);
begin
 FPassword:=Value;
 changed(False);
end;

procedure TRpDbIBInfo.SetConnectionAlias(Value:string);
begin
 if length(Trim(Value))<1 then
  Raise Exception.Create(SRpDatabaseAliasNull);
 if FConnectionAlias=Value then
  exit;
 if collection=nil then
 begin
  FConnectionAlias:=Uppercase(Value);
  exit;
 end;
 if (csloading in (collection As TRpDBIBList).FDatabaseconf.COmponentState) then
 begin
  FConnectionAlias:=Uppercase(Value);
  exit;
 end;
 if TRpDBIBList(collection).Indexof(Value)>=0 then
  Raise Exception.Create(SRpAliasExists+Value);
 FConnectionAlias:=Uppercase(Value);
 changed(False);
end;


function TRpDbIBInfo.GetIBXFilename:string;
begin
 Result:=BuildIBDatabaseName(remote,protocol,server,filename);
end;

procedure TRpDbIBInfo.SetConnected(Value:boolean);
begin
 FConnected:=Value;
 changed(False);
end;

procedure TRpDbIBInfo.SetLoginPrompt(Value:boolean);
begin
 FLoginprompt:=Value;
 changed(False);
end;


procedure TRpDbIBInfo.SetServer(Value:string);
begin
 FServer:=Value;
 changed(False);
end;

procedure TRpDbIBInfo.SetFilename(Value:string);
begin
 FFilename:=Value;
 changed(False);
end;

procedure TRpDbIBInfo.SetCharacterset(Value:string);
begin
 FCharacterSet:=Value;
 changed(False);
end;

procedure TRpDbIBInfo.SetRole(Value:string);
begin
 FRole:=Value;
 changed(False);
end;

procedure TRpDbIBInfo.SetRemote(Value:boolean);
begin
 FRemote:=Value;
 changed(False);
end;

procedure TRpDbIBInfo.SetProtocol(Value:TRpIBXProtocol);
begin
 FProtocol:=Value;
 changed(False);
end;

procedure TRpDbIBInfo.SetSQLDialect(Value:integer);
begin
 FSQLDialect:=Value;
 changed(False);
end;

procedure TRpDbIBInfo.SetDatabase(Value:TIBDatabase);
begin
 if (csloading in (collection As TRpDBIBList).FDatabaseconf.COmponentState) then
 begin
  FDatabase:=Value;
  exit;
 end;
 FDatabase:=Value;
 changed(False);
end;

// TRpDBIBList

Constructor TRpDBIBList.Create(dbconf:TRpDBIBConf);
begin
 inherited Create(TRpDbIBInfo);
 FDatabaseConf:=dbconf;
end;

function TRpDBIBList.GetItem(Index:Integer):TRpDbIBInfo;
begin
 Result:=TRpDbIBInfo(inherited GetItem(index));
end;

procedure TRpDBIBList.SetItem(index:integer;Value:TRpDbIBInfo);
begin
 inherited SetItem(Index,Value);
end;

procedure TRpDBIBList.AssignTo(Dest:TPersistent);
var
 i:integer;
 ds:TStrings;
begin
 if (Dest is TStrings) then
 begin
  ds:=TStrings(Dest);
  ds.clear;
  for i:=0 to count-1 do
  begin
   ds.Add(items[i].ConnectionAlias);
  end;
 end
 else
  inherited AssignTo(Dest);
end;

function TRpDBIBList.IndexOf(ConnectionAlias:string):integer;
var
 i:integer;
begin
 Result:=-1;
 i:=0;
 While i<count do
 begin
  if items[i].ConnectionAlias=Uppercase(ConnectionAlias) then
  begin
   Result:=i;
   break;
  end;
  inc(i);
 end;
end;

function TRpDBIBList.Add(ConnectionAlias:string):TRpDbIBInfo;
begin
 if length(Trim(ConnectionAlias))<1 then
  Raise Exception.Create(SRpDatabaseAliasNull);
 if indexof(Trim(ConnectionAlias))>=0 then
  Raise Exception.Create(SRpAliasExists+ConnectionAlias);
 Result:=TRpDbIBInfo(inherited Add);
 Result.FConnectionAlias:=Uppercase(Trim(ConnectionAlias));
end;

// TRpDBIBConf
constructor TRpDBIBConf.Create(AOwner:TComponent);
begin
 inherited Create(AOwner);

 FDatabases:=TRpDBIBList.create(self);
 FSystemConfig:=True;
end;

destructor TRpDBIBConf.Destroy;
begin
 FDatabases.free;

 inherited Destroy;
end;


procedure TRpDBIBConf.Loaded;
begin
 inherited Loaded;
 // Loads the database configuration from the registry
 // And opens the autoopen databases

end;

procedure TRpDBIBConf.Notification(AComponent:TComponent;Operation:TOperation);
var i:integer;
begin
 inherited Notification(AComponent,Operation);
 if Operation=OpRemove then
 begin
  if (AComponent is TIBDatabase) then
  begin
   with FDatabases do
   begin
    for i:=0 to Count -1 do
    begin
     if items[i]<>nil then
     begin
      if Items[i].Database=AComponent then
       Items[i].Database:=nil;
     end;
    end;
   end;
  end;
 end;
end;

procedure TRpDBIBConf.SetDatabases(Value:TRpDBIBList);
begin
 FDatabases.assign(Value);
end;



procedure TFRpIBDatabases.FormCreate(Sender: TObject);
{$IFDEF MSWINDOWS}
var
 x:integer;
{$ENDIF}
begin
 comboconnectionvisible:=True;
 databaseconf:=TRpDBIBConf.create(Self);
 langnames:=TStringlist.create;
 langdesc:=TStringlist.create;
 langchar:=TStringlist.create;

{$IFDEF MSWINDOWS}
 x:=GetDeviceCaps(GetDC(0),LOGPIXELSX)*2;
 SendMessage(ComboDatabase.handle,CB_SETDROPPEDWIDTH ,x,0);
{$ENDIF}
end;

procedure TFRpIBDatabases.FormDestroy(Sender: TObject);
begin
 langnames.free;
 langdesc.free;
 langchar.free;
end;

{$IFDEF MSWINDOWS}
// TRpDBIBListPropEditor
function TRpDBIBListPropEditor.GetAttributes:TPropertyAttributes;
begin
 Result:=[paDialog];
end;

procedure TRpDBIBListPropEditor.Edit;
var
 Dia:TFRpIBDatabases;
 i:integer;
begin
 Dia:=TFRpIBDatabases.Create(Application);
 try
  dia.helpcontext:=(GetComponent(0) As TRpDBIBConf).Helpcontext;
  Dia.Designer:=Designer;
  Dia.databaseconf.databases:=(GetComponent(0) As TRpDBIBConf).Databases;
  dia.databaseconf.Systemconfig:=(GetComponent(0) As TRpDBIBConf).Systemconfig;
  dia.databaseconf.Aliasconfig:=(GetComponent(0) As TRpDBIBConf).AliasConfig;
  if (Dia.ShowModal=mrOK) then
  begin
   for i:=0 to PropCount-1 do
   begin
    (GetComponent(i) As TRpDBIBConf).Databases:=Dia.databaseconf.Databases;
   end;
   Modified;
  end;
 finally
  Dia.free;
 end;
end;

function TRpDBIBListPropEditor.GetValue:string;
begin
 Result:='Database List';
end;
{$ENDIF}
procedure TRpDBIBConf.ShowConfig(AOwner:TCOmponent);
var
 Dia:TFRpIBDatabases;
 i:integer;
begin
 Dia:=TFRpIBDatabases.Create(Application);
 try
  dia.helpcontext:=helpcontext;
  Dia.databaseconf.databases:=Databases;
  Dia.AOwner:=dia;
  dia.databaseconf.Systemconfig:=Systemconfig;
  dia.databaseconf.Aliasconfig:=AliasConfig;
  // Creates the databases
  for i:=0 to dia.databaseconf.Databases.count-1 do
  begin
   dia.databaseconf.databases.items[i].database:=nil;
   dia.databaseconf.databases.items[i].CreateDatabase(dia);
  end;
  if (Dia.ShowModal=mrOK) then
  begin
   // Remove old databases
   for i:=0 to databases.count-1 do
   begin
    databases.Items[i].database.free;
    databases.items[i].database:=nil;
   end;
   // Insert new databases
   Databases:=Dia.databaseconf.Databases;
   for i:=0 to databases.count-1 do
   begin
    dia.removecomponent(databases.items[i].database);
    AOwner.insertcomponent(databases.items[i].database);
   end;
  end;
 finally
  Dia.free;
 end;
end;

procedure TRpDBIBConf.ShowConfig(AllowLoadandsave,AllowNewDelete,autosave:boolean);
var
 Dia:TFRpIBDatabases;
begin
 Dia:=TFRpIBDatabases.Create(Application);
 try
  dia.helpcontext:=helpcontext;
  Dia.databaseconf.databases:=Databases;
  Dia.AllowLoadAndsave:=AllowLoadAndsave;
  dia.databaseconf.Systemconfig:=Systemconfig;
  dia.databaseconf.Aliasconfig:=AliasConfig;
  Dia.AllowNewDelete:=AllowNewDelete;
  if (Dia.ShowModal=mrOK) then
  begin
   Databases:=Dia.databaseconf.Databases;
   if autosave then
    dia.BSaveClick(dia);
  end;
 finally
  Dia.free;
 end;
end;


procedure TFRpIBDatabases.FormShow(Sender: TObject);
begin
 // Loks if a component to create databases
 if Assigned(AOwner) then
 begin
  LDatabase.visible:=False;
  Combodatabase.Visible:=False;
 end;

 // Reload list
 Reloadlist;
end;

procedure TFRpIBDatabases.Reloadlist;
var
 i:integer;
begin
 DBList.clear;
 if databaseconf.Databases.count<1 then
 begin
  PConect.visible:=false;
 end
 else
 begin
  PConect.visible:=True;
  for i:=0 to databaseconf.databases.count-1 do
  begin
   dblist.items.add(databaseconf.databases.Items[i].ConnectionAlias);
  end;
  dblist.itemindex:=0;
  UpdateAlias;
 end;
{$IFDEF MSWINDOWS}
 if Assigned(Designer) then
 begin
  LDatabase.Visible:=True;
  CheckConnected.visible:=True;
  ComboDatabase.Visible:=True;
  BNew.Visible:=True;
  BRemove.Visible:=True;
  BRename.Visible:=True;
  BLoad.Visible:=True;
  BSave.Visible:=True;
 end
 else
{$ENDIF}
 begin
  LDatabase.Visible:=False;
  CheckConnected.visible:=false;
  ComboDatabase.Visible:=False;
  if Assigned(AOwner) then
  begin
   BNew.Visible:=True;
   BRemove.Visible:=True;
   BRename.Visible:=True;
   Bload.Visible:=True;
   BSave.Visible:=True;
  end
  else
  begin
   BLoad.Visible:=AllowLoadAndSave;
   BSave.Visible:=AllowLoadAndSave;
   BNew.Visible:=AllowNewDelete;
   BRemove.Visible:=AllowNewDelete;
   BRename.Visible:=AllowNewDelete;
  end;
 end;
end;

procedure TFRpIBDatabases.UpdateAlias;
begin
 loading:=true;
 try
  if (dblist.itemindex<0) and (dblist.items.count>0) then
   dblist.itemindex:=0;
  // Assign parameters
  if dblist.items.count>0 then
  begin
   with databaseconf.databases.items[dblist.itemindex] do
   begin
    // Search the driver type
    EUsername.text:=Username;
    EPassword.text:=Password;
    CheckLoginPrompt.checked:=LoginPrompt;
    CheckConnected.checked:=Connected;
    // IBX
    EServer.Text:=Server;
    EFilename.text:=Filename;
    ComboProtocol.ItemIndex:=Integer(Protocol);
    if SQLDialect=3 then
     ComboVersion.itemindex:=1
    else
     ComboVersion.itemindex:=0;
    ComboCharset.itemindex:=Combocharset.items.indexof(Characterset);
    ERole.text:=Role;
    RadioLocal.checked:=Not Remote;
    RadioRemote.checked:=Remote;
    LocalRemoteChanged;
    ConnectionTypeChanged;
   end;
  end;
 finally
  loading:=false;
 end;
end;



function BuildIBDatabaseName(Remote:Boolean;Protocol:TRpIBXProtocol;Servername,Filename:string):string;
begin
 if not remote then
 begin
  Result:=filename;
  exit;
 end;
 case Protocol of
  rpIBXTCP:
   begin
    Result:=Servername+':'+Filename;
   end;
  rpIBXNetBeui:
   begin
    Result:='\\'+Servername+'\'+Filename;
   end;
  rpIBXIpx:
   begin
    Result:=Servername+'@'+Filename;
   end;
 end;
end;

function TFRpIBDatabases.FindDatabaseInfo:TRpDbIBInfo;
var
 lindex:integer;
begin
 result:=nil;
 if dblist.itemindex<0 then
  exit;
 lindex:=databaseconf.Databases.IndexOf(dblist.items.strings[dblist.itemindex]);
 if lindex<0 then
  exit;
 Result:=databaseconf.databases.items[lindex];
end;


procedure TFRpIBDatabases.ConnectionTypeChanged;
begin
 // Updates the database
 UpdateComboDatabase;
 // Show only selected tab
end;


procedure TFRpIBDatabases.LocalRemoteChanged;
begin
 EServer.visible:=RadioRemote.checked;
 LServer.visible:=RadioRemote.checked;
 ComboProtocol.visible:=RadioRemote.checked;
 LProtocol.visible:=RadioRemote.checked;
end;

procedure TFRpIBDatabases.RadioLocalClick(Sender: TObject);
var
 dbinfo:TRpDbIBInfo;
begin
 dbinfo:=FindDatabaseinfo;
 if dbinfo=nil then
  Exit;
 dbinfo.remote:=Not RadioLocal.checked;
 if Not dbinfo.remote then
 begin
  dbinfo.server:='';
  EServer.text:='';
 end;
 LocalRemoteChanged;
 if not loading then achange:=true;;
end;

procedure TFRpIBDatabases.EServerChange(Sender: TObject);
var
 dbinfo:TRpDbIBInfo;
begin
 dbinfo:=FindDatabaseinfo;
 if dbinfo=nil then
  exit;
 dbinfo.Server:=EServer.text;
 if not loading then achange:=true;;
end;

procedure TFRpIBDatabases.ComboProtocolChange(Sender: TObject);
var
 dbinfo:TRpDbIBInfo;
begin
 dbinfo:=FindDatabaseinfo;
 if dbinfo=nil then
  exit;
 dbinfo.Protocol:=TRpIBXProtocol(ComboProtocol.itemindex);
 if not loading then achange:=true;;
end;

procedure TFRpIBDatabases.EFilenameChange(Sender: TObject);
var
 dbinfo:TRpDbIBInfo;
begin
 dbinfo:=FindDatabaseinfo;
 if dbinfo=nil then
  exit;
 dbinfo.Filename:=EFilename.text;
 if not loading then achange:=true;;
end;

procedure TFRpIBDatabases.ComboCharsetChange(Sender: TObject);
var
 dbinfo:TRpDbIBInfo;
begin
 dbinfo:=FindDatabaseinfo;
 if dbinfo=nil then
  exit;
 dbinfo.CharacterSet:=ComboCharset.text;
 if not loading then achange:=true;;
end;

procedure TFRpIBDatabases.EroleChange(Sender: TObject);
var
 dbinfo:TRpDbIBInfo;
begin
 dbinfo:=FindDatabaseinfo;
 if dbinfo=nil then
  exit;
 dbinfo.Role:=Erole.text;
 if not loading then achange:=true;;
end;

procedure TFRpIBDatabases.EUsernameChange(Sender: TObject);
var
 dbinfo:TRpDbIBInfo;
begin
 dbinfo:=FindDatabaseinfo;
 if dbinfo=nil then
  exit;
 dbinfo.Username:=EUsername.text;
 if not loading then achange:=true;;
end;

procedure TFRpIBDatabases.EPasswordChange(Sender: TObject);
var
 dbinfo:TRpDbIBInfo;
begin
 dbinfo:=FindDatabaseinfo;
 if dbinfo=nil then
  exit;
 dbinfo.Password:=EPassword.text;
 if not loading then achange:=true;;
end;

procedure TFRpIBDatabases.CheckLoginPromptClick(Sender: TObject);
var
 dbinfo:TRpDbIBInfo;
begin
 dbinfo:=FindDatabaseinfo;
 if dbinfo=nil then
  exit;
 dbinfo.LoginPrompt:=CheckLoginPrompt.checked;
 if not loading then achange:=true;;
end;


procedure TFRpIBDatabases.CheckConnectedClick(Sender: TObject);
var
 dbinfo:TRpDbIBInfo;
begin
 dbinfo:=FindDatabaseinfo;
 if dbinfo=nil then
  exit;
 dbinfo.Connected:=CheckConnected.checked;
 if not loading then achange:=true;;
end;



procedure TFRpIBDatabases.UpdateComboDatabase;
var
 dbinfo:TRpDbIBInfo;
{$IFDEF MSWINDOWS}
 i:integer;
{$ENDIF}
begin
{$IFDEF MSWINDOWS}
 if Not Assigned(Designer) then
  exit;
{$ENDIF}
 ComboDatabase.items.clear;
 ComboDatabase.items.add('');
 // Fills the combo and sets the itemindex on it
{$IFDEF MSWINDOWS}
 Designer.GetComponentnames(GetTypeData(TypeInfo(TIBDatabase)),Addname);
{$ENDIF}
 dbinfo:=FindDatabaseinfo;
 if dbinfo=nil then
 begin
  combodatabase.ItemIndex:=-1;
  exit;
 end;
{$IFDEF MSWINDOWS}
 if Assigned(dbinfo.database) then
 begin
  i:=0;
  While i<Combodatabase.items.count do
  begin
   if dbinfo.database=Designer.GetComponent(combodatabase.items.strings[i]) then
   begin
    ComboDatabase.itemindex:=i;
    break;
   end;
   inc(i);
  end;
 end
 else
 begin
  combodatabase.ItemIndex:=-1;
  exit;
 end;
{$ENDIF}
end;

procedure TFRpIBDatabases.Addname(const aname:string);
begin
 ComboDatabase.Items.Add(aname);
end;

procedure TFRpIBDatabases.ComboDatabaseClick(Sender: TObject);
var
 dbinfo:TRpDbIBInfo;
begin
 dbinfo:=FindDatabaseInfo;
 if dbinfo=nil then
  exit;
{$IFDEF MSWINDOWS}
 if not assigned(designer) then
  exit;

 if Combodatabase.itemindex<1 then
  dbinfo.database:=nil
 else
 begin
  dbinfo.database:=TIBDatabase(Designer.getcomponent(combodatabase.items.strings[combodatabase.itemindex]));
 end;
{$ENDIF}
end;

procedure TFRpIBDatabases.BNewClick(Sender: TObject);
var
 dbinfo,info:TRpDbIBInfo;
 aname:string;
begin
 dbinfo:=FindDatabaseInfo;
 aname:=InputBox(SRpNewDatabaseconf,SRpEnterTheName,'');
 info:=databaseconf.databases.add(aname);
 if Assigned(dbinfo) then
 begin
  info.UserName:=dbinfo.Username;
  info.Password:=dbinfo.Password;
  info.LoginPrompt:=dbinfo.LoginPrompt;
  info.Connected:=dbinfo.connected;
  info.Protocol:=dbinfo.protocol;
  info.remote:=dbinfo.remote;
  info.server:=dbinfo.server;
 end;
 if Assigned(AOwner) then
 begin
  info.createDatabase(AOwner);
 end;
 ReloadList;
 achange:=true;;
 dblist.itemindex:=dblist.items.count-1;
 dblistclick(Self);
end;

procedure TFRpIBDatabases.DBListClick(Sender: TObject);
begin
 UpdateAlias;
end;

procedure TFRpIBDatabases.BRemoveClick(Sender: TObject);
var
 dbinfo:TRpDbIBInfo;
begin
 dbinfo:=FindDatabaseInfo;
 if dbinfo=nil then
  exit;
 if assigned(AOwner) then
  if assigned(dbinfo.database) then
   if (dbinfo.database.owner=AOwner) then
    dbinfo.database.free;
 databaseconf.Databases.Delete(dbinfo.Index);
 achange:=true;
 ReloadList;
end;

procedure TFRpIBDatabases.BRenameClick(Sender: TObject);
var
 dbinfo:TRpDbIBInfo;
 aname:string;
begin
 dbinfo:=FindDatabaseInfo;
 if dbinfo=nil then
  exit;
 aname:=InputBox(SRpChangeAliasConf,SRpEnterTheNewName,dbinfo.ConnectionAlias);
 dbinfo.ConnectionAlias:=aname;
 dblist.items.strings[dblist.itemindex]:=aname;
 achange:=true;
end;

procedure TFRpIBDatabases.BUpClick(Sender: TObject);
var
 cconf:TRpDBIBConf;
 i,newindex:integer;
 info:TRpDbIBInfo;
begin
 if dblist.itemindex<1 then
  exit;
 newindex:=dblist.itemindex-1;
 cconf:=TRpDBIBConf.create(Application);
 try
  cconf.Databases.assign(databaseconf.databases);
  While databaseconf.databases.count>0 do
  begin
   databaseconf.databases.delete(0);
  end;
  for i:=0 to newindex-1 do
  begin
   info:=databaseconf.databases.add(cconf.databases.items[i].ConnectionAlias);
   info.assign(cconf.databases.items[i]);
  end;
  info:=databaseconf.databases.add(cconf.databases.items[newindex+1].ConnectionAlias);
  info.assign(cconf.databases.items[newindex+1]);
  info:=databaseconf.databases.add(cconf.databases.items[newindex].ConnectionAlias);
  info.assign(cconf.databases.items[newindex]);
  for i:=newindex+2 to cconf.databases.count-1 do
  begin
   info:=databaseconf.databases.add(cconf.databases.items[i].ConnectionAlias);
   info.assign(cconf.databases.items[i]);
  end;
 finally
  cconf.free;
 end;
 ReloadList;
 DBList.itemindex:=newindex;
 dblistclick(Self);
end;

procedure TFRpIBDatabases.BDownClick(Sender: TObject);
var
 cconf:TRpDBIBConf;
 i,newindex:integer;
 info:TRpDbIBInfo;
begin
 if dblist.itemindex>dblist.items.count-1 then
  exit;
 newindex:=dblist.itemindex;
 cconf:=TRpDBIBConf.create(Application);
 try
  cconf.Databases.assign(databaseconf.databases);
  While databaseconf.databases.count>0 do
  begin
   databaseconf.databases.delete(0);
  end;
  for i:=0 to newindex-1 do
  begin
   info:=databaseconf.databases.add(cconf.databases.items[i].ConnectionAlias);
   info.assign(cconf.databases.items[i]);
  end;
  info:=databaseconf.databases.add(cconf.databases.items[newindex+1].ConnectionAlias);
  info.assign(cconf.databases.items[newindex+1]);
  info:=databaseconf.databases.add(cconf.databases.items[newindex].ConnectionAlias);
  info.assign(cconf.databases.items[newindex]);
  for i:=newindex+2 to cconf.databases.count-1 do
  begin
   info:=databaseconf.databases.add(cconf.databases.items[i].ConnectionAlias);
   info.assign(cconf.databases.items[i]);
  end;
 finally
  cconf.free;
 end;
 ReloadList;
 DBList.itemindex:=newindex+1;
 dblistclick(Self);
end;

procedure TRpDBIBConf.SetConnected(Value:Boolean);
var
 i:integer;
begin
 if Value=FConnected then
  exit;
 if csloading in componentstate then
 begin
  FCOnnected:=Value;
  exit;
 end;
 if Value then
 begin
  for i:=0 to databases.count-1 do
  begin
   if databases.items[i].Connected then
   begin
    if Assigned(databases.items[i].database) then
     databases.items[i].ConnectDatabase;
   end;
  end;
 end
 else
 begin
  for i:=0 to databases.count-1 do
  begin
   if databases.items[i].Connected then
   begin
    if Assigned(databases.items[i].database) then
     databases.items[i].DisConnectDatabase;
   end;
  end;
 end;
 FCOnnected:=Value;
end;

procedure TRpDbIBInfo.DisconnectDatabase;
begin
 if Not assigned(FDatabase) then
  exit;
 TIBDatabase(FDatabase).connected:=False;
end;

function TRpDbIBInfo.ConnectDatabase:Boolean;
begin
 DisconnectDatabase;
 Result:=False;
 if Not Assigned(FDatabase) then
 begin
  exit;
 end;
 ConnectIBX(TIBDatabase(FDatabase));
 Result:=True;
end;


procedure TRpDbIBInfo.ConnectIBX(dbase:TIBDatabase);
begin
 dbase.loginprompt:=Floginprompt;
 dbase.DatabaseName:=BuildIBDatabaseName(FRemote,FProtocol,FServer,FFilename);
 with dbase.params do
 begin
  Clear;
  Add('user_name='+FUsername);
  Add('password='+FPassword);
  Add('lc_ctype='+FCharacterset);
  Add('sql_role_name='+FRole);
 end;
 dbase.SQLDialect:=SQLDialect;
 dbase.connected:=True;
end;

procedure TFRpIBDatabases.ComboVersionClick(Sender: TObject);
var
 dbinfo:TRpDbIBInfo;
begin
 dbinfo:=FindDatabaseinfo;
 if dbinfo=nil then
  exit;
 if ComboVersion.itemindex=0 then
  dbinfo.SQLDialect:=1
 else
  dbinfo.SQLDialect:=3;
 if not loading then achange:=true;;
end;

procedure TFRpIBDatabases.BTestClick(Sender: TObject);
var
 dbinfo:TRpDbIBInfo;
begin
 dbinfo:=FindDatabaseinfo;
 if dbinfo=nil then
  exit;
 dbinfo.DisconnectDatabase;
 try
  if assigned(dbinfo.database) then
  begin
   dbinfo.ConnectDatabase;
   ShowMessage(SRpConnectionsuccesfull);
  end
  else
  begin
   dbinfo.CreateDatabase(Application);
   try
    dbinfo.ConnectDatabase;
    ShowMessage(SRpConnectionsuccesfull);
   finally
    dbinfo.database.Free;
    dbinfo.database:=nil;
   end;
  end;
 finally
  dbinfo.DisconnectDatabase;
 end;
end;

procedure TRpDBIBConf.LoadConfig;
var
 i:integer;
 ininame:string;
 inifile:TInifile;
begin
 if FSystemconfig then
  ininame:=Obtainininamecommonconfig('','ibaccess','databases')
 else
  ininame:=Obtainininameuserconfig('','ibaccess','databases');
 inifile:=TInifile.create(ininame);
 try
  for i:=0 to Databases.count-1 do
  begin
   databases.items[i].LoadConfig(FConfigurationIndex,inifile);
  end;
 finally
  inifile.free;
 end;
end;

procedure TRpDBIBConf.SaveConfig;
var
 i:integer;
 ininame:string;
 inifile:TInifile;
begin
 if FSystemconfig then
  ininame:=Obtainininamecommonconfig('','ibaccess','databases')
 else
  ininame:=Obtainininameuserconfig('','ibaccess','databases');

 inifile:=TInifile.create(ininame);
 try
  for i:=0 to Databases.count-1 do
  begin
   databases.items[i].SaveConfig(FConfigurationIndex,inifile);
  end;
  inifile.UpdateFile;
 finally
  inifile.free;
 end;
end;

procedure TRpDBIBConf.SaveAliases;
var
 i:integer;
 ininame:string;
 inifile:TInifile;
begin
 if FSystemconfig then
  ininame:=Obtainininamecommonconfig('','ibaccess','databases')
 else
  ininame:=Obtainininameuserconfig('','ibaccess','databases');
 inifile:=TInifile.create(ininame);
 try
  inifile.WriteInteger('Aliases'+IntToStr(ConfigurationIndex),'AliasNum',databases.count);
  for i:=0 to Databases.count-1 do
  begin
   inifile.WriteString('Aliases'+IntToStr(ConfigurationIndex),'Alias'+IntToStr(i),databases.items[i].ConnectionAlias);
   databases.Items[i].SaveConfig(configurationindex,inifile);
  end;
  inifile.UpdateFile;
 finally
  inifile.free;
 end;
end;

procedure TRpDBIBConf.LoadAliases;
var
 i:integer;
 ininame:string;
 aliascount:integer;
 index:integer;
 inifile:TInifile;
 newalias:string;
begin
 if FSystemconfig then
  ininame:=Obtainininamecommonconfig('','ibaccess','databases')
 else
  ininame:=Obtainininameuserconfig('','ibaccess','databases');
 While databases.count>0 do
  databases.delete(0);
 inifile:=TInifile.create(ininame);
 try
  aliascount:=inifile.ReadInteger('Aliases'+IntToStr(ConfigurationIndex),'AliasNum',0);
  for i:=0 to aliascount-1 do
  begin
   newalias:=inifile.ReadString('Aliases'+IntToStr(ConfigurationIndex),'Alias'+IntToStr(i),'');
   if Length(NewAlias)>0 then
   begin
    index:=databases.IndexOf(newalias);
    if index<0 then
    begin
     databases.Add(newalias);
     index:=databases.count-1;
    end;
    databases.items[index].LoadConfig(ConfigurationIndex,inifile);
   end;
  end;
 finally
  inifile.free;
 end;
end;

procedure TRpDbIBInfo.LoadConfig(Index:integer;inifile:TInifile);
var
 section:string;
begin
 section:=FConnectionAlias+inttostr(index);
 with Inifile do
 begin
  FloginPrompt:=ReadBool(section,'LoginPrompt',false);
  FConnected:=ReadBool(section,'Connected',True);
  FUsername:=ReadString(section,'Username','');
  FPassword:=ReadString(section,'Password','');
  // IBX
  FServer:=ReadString(section,'Server','');
  FFilename:=ReadString(section,'Filename','');
  FCharacterset:=ReadString(section,'Characterset','WIN1252');
  FRemote:=ReadBool(section,'Remote',false);
  FRole:=ReadString(section,'Role','');
  FProtocol:=TRpIBXProtocol(ReadInteger(section,'Protocol',0));
  FSQLDialect:=ReadInteger(section,'SQLDialect',1);
 end;
 Changed(false);
end;

procedure TRpDbIBInfo.SaveConfig(Index:integer;inifile:TInifile);
var
 section:string;
begin
 section:=FConnectionAlias+inttostr(index);
 with Inifile do
 begin
  WriteInteger(section,'ConnectionType',Integer(FConnectionType));
  WriteBool(section,'LoginPrompt',FloginPrompt);
  WriteBool(section,'Connected',FConnected);
  WriteString(section,'Username',FUsername);
  WriteString(section,'Password',FPassword);
  // IBX
  WriteString(section,'Server',FServer);
  WriteString(section,'Filename',FFilename);
  WriteString(section,'Characterset',FCharacterset);
  WriteBool(section,'Remote',FRemote);
  WriteString(section,'Role',FRole);
  WriteInteger(section,'Protocol',integer(FProtocol));
  WriteInteger(section,'SQLDialect',FSQLDialect);
 end;
end;

procedure TFRpIBDatabases.BLoadClick(Sender: TObject);
begin
 if databaseconf.aliasconfig then
  databaseconf.LoadAliases
 else
  databaseconf.LoadConfig;
 Reloadlist;
end;

procedure TFRpIBDatabases.BSaveClick(Sender: TObject);
begin
 if databaseconf.aliasconfig then
  databaseconf.SaveAliases
 else
  databaseconf.SaveConfig;
end;


procedure TRpDbIBInfo.GetTableNames(Tablenames:TStrings);
begin
 if Not Assigned(fdatabase) then
  exit;
 TIBDatabase(FDatabase).GetTableNames(tablenames)
end;


procedure TFRpIBDatabases.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
var
 res:TModalresult;
begin
 if achange then
 begin
  res:=MEssageDlg(SRpSaveChanges,mtwarning,[mbyes,mbno,mbcancel],0);
  if res=mryes then
   modalresult:=mrok
  else
   if res=mrcancel then
    canclose:=false;
 end;
end;

initialization
end.
