{*******************************************************}
{                                                       }
{       rpshfolder                                      }
{                                                       }
{       Author:                                         }
{       Toni Martir                                     }
{       http://www.ibaccess.org                         }
{       e-mail: toni@pala.com                           }
{                                                       }
{       An interface to user and system config files    }
{       to store information with a Windows 2000        }
{       compliant procedure                             }
{                                                       }
{       OpenSource license (MPL)                        }
{                                                       }
{       This file needs the pascal transation of        }
{       shfolder.h, thanks to Rudy Velthuis for this    }
{       translation                                     }
{                                                       }
{*******************************************************}

unit rpshfolder;

interface


uses
  SysUtils, Classes,
{$IFDEF LINUX}
  Libc,
{$ENDIF}
  rpdconsts;


  function Obtainininameuserconfig(company,product,filename:string):string;
  function Obtainininamelocalconfig(company,product,filename:string):string;
  function Obtainininamecommonconfig(company,product,filename:string):string;

implementation



function Obtainininameuserconfig(company,product,filename:string):string;
var
 szAppData:array [0..MAX_PATH] of char;
 ap:PCHar;
begin
{$IFDEF LINUX}
 ap:=getenv(Pchar('HOME'));
 if assigned(ap) then
 begin
  StrPCopy(szAppdata,ap);
  Result:=StrPas(szAppdata)+'/.'
 end
 else
  Result:='./.';
 if length(company)>0 then
  Result:=Result+company+'.';
 if length(product)>0 then
  Result:=Result+product+'.';
 Result:=Result+filename;
{$ENDIF}
{$IFDEF MSWINDOWS}
 if length(filename)<1 then
  Raise Exception.Create(SRpFileNameRequired);
 SHGetFolderPath(0, CSIDL_APPDATA or CSIDL_FLAG_CREATE, 0, 0, szAppData);
 if length(company)>0 then
 begin
  if not PathAppend(szAppdata,Pchar(company)) then
   RaiseLastWin32Error;
 end;
 if Length(product)>0 then
 begin
  if not PathAppend(szAppdata,Pchar(product)) then
   RaiseLastWin32Error;
 end;
 Result:=StrPas(szAppdata);
 if Not DirectoryExists(Result) then
 begin
  if not CreateDirectory(Pchar(Result),nil) then
   Result:='';
 end;
 if not PathAppend(szAppdata,Pchar(filename+'.ini')) then
  RaiseLastWin32Error;
 Result:=StrPas(szAppdata);
{$ENDIF}
end;


function Obtainininamelocalconfig(company,product,filename:string):string;
{$IFDEF MSWINDOWS}
var
 szAppData:array [0..MAX_PATH] of char;
{$ENDIF}
begin
{$IFDEF LINUX}
 Result:='/etc/'+company+product+filename;
{$ENDIF}
{$IFDEF MSWINDOWS}
 if length(filename)<1 then
  Raise Exception.Create(SRpFileNameRequired);
 SHGetFolderPath(0, CSIDL_LOCAL_APPDATA or CSIDL_FLAG_CREATE, 0, 0, szAppData);

 if length(company)>0 then
 begin
  if not PathAppend(szAppdata,Pchar(company)) then
   RaiseLastWin32Error;
 end;
 if Length(product)>0 then
 begin
  if not PathAppend(szAppdata,Pchar(product)) then
   RaiseLastWin32Error;
 end;
 Result:=StrPas(szAppdata);
 if Not DirectoryExists(Result) then
 begin
  if not ForceDirectories(Result) then
   Result:='';
 end;
 if not PathAppend(szAppdata,Pchar(filename+'.ini')) then
  RaiseLastWin32Error;
 Result:=StrPas(szAppdata);
{$ENDIF}
end;


function Obtainininamecommonconfig(company,product,filename:string):string;
{$IFDEF MSWINDOWS}
var
 szAppData:array [0..MAX_PATH] of char;
{$ENDIF}
begin
{$IFDEF LINUX}
 Result:='/etc/'+company+product+filename;
{$ENDIF}
{$IFDEF MSWINDOWS}
 if length(filename)<1 then
  Raise Exception.Create(SRpFileNameRequired);
 SHGetFolderPath(0, CSIDL_COMMON_APPDATA or CSIDL_FLAG_CREATE, 0, 0, szAppData);
 if length(company)>0 then
 begin
  if not PathAppend(szAppdata,Pchar(company)) then
   RaiseLastWin32Error;
 end;
 if Length(product)>0 then
 begin
  if not PathAppend(szAppdata,Pchar(product)) then
   RaiseLastWin32Error;
 end;
 Result:=StrPas(szAppdata);
 if Not DirectoryExists(Result) then
 begin
  if not ForceDirectories(Result) then
   Result:='';
 end;
 if not PathAppend(szAppdata,Pchar(filename+'.ini')) then
  RaiseLastWin32Error;
 Result:=StrPas(szAppdata);
{$ENDIF}
end;

initialization


end.
