{*******************************************************}
{                                                       }
{       IBAccess                                        }
{       Service progres  dialog                         }
{                                                       }
{       Author:                                         }
{       Toni Martir                                     }
{       http://www.ibaccess.org                         }
{       e-mail: toni@pala.com                           }
{                                                       }
{       OpenSource license (MPL)                        }
{                                                       }
{*******************************************************}

unit ibproces;

interface

uses
  SysUtils, Classes, QForms, ibservices,
  ibacconst, QDialogs, QImgList, QActnList, QMenus, QTypes, QControls,
  QStdCtrls, QComCtrls;

type

  TFIBProces = class(TForm)
    MainMenu1: TMainMenu;
    ActionList1: TActionList;
    File1: TMenuItem;
    ASave: TAction;
    IconImages: TImageList;
    AClose: TAction;
    ACopy: TAction;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    Edit1: TMenuItem;
    ASave1: TMenuItem;
    N1: TMenuItem;
    Close1: TMenuItem;
    ACopy1: TMenuItem;
    SaveDialog1: TSaveDialog;
    Memo: TMemo;
    procedure ACloseExecute(Sender: TObject);
    procedure ACopyExecute(Sender: TObject);
    procedure ASaveExecute(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
    ibservice:TIBControlAndQueryService;
    oldidle:TIdleEvent;
    procedure Appidle(Sender:TObject;var done:Boolean);
{$IFDEF MSWINDOWS}
    procedure TTerminate(Sender:TObject);
{$ENDIF}
  public
    { Public declarations }
    isterminated:boolean;
  end;

  TIBProcesThread=class(TThread)
   private
    FIBService:TIBControlAndQueryService;
    Msg:string;
    form:TFIBProces;
    procedure SyncMessage;
   public
    procedure Execute;override;
  end;
procedure COntrolIBService(ibservice:TIBControlAndQueryService);

implementation

{$R *.xfm}

procedure COntrolIBService(ibservice:TIBControlAndQueryService);
var
 dia:TFIBProces;
begin
 dia:=TFIBProces.create(Application);
 try
  dia.ibservice:=ibservice;
  dia.oldidle:=application.onidle;
  application.onidle:=dia.Appidle;
  try
   dia.showmodal;
  finally
   application.onidle:=dia.oldidle;
  end;
 finally
  dia.free;
 end;
end;

procedure TFIBProces.Appidle(Sender:TObject;var done:Boolean);
{$IFDEF MSWINDOWS}
var
 proces:TIBProcesThread;
{$ENDIF}
begin
 Done:=False;
 Application.onidle:=nil;
 // For Linux multithread sincronization does not
 // work when not happens in the main form
 {$IFDEF MSWINDOWS}
 proces:=TIBProcesThread.Create(True);
 proces.FIBService:=ibservice;
 proces.form:=self;
 proces.OnTerminate:=TTerminate;
 proces.FreeOnTerminate:=True;
 proces.resume;
 {$ENDIF}
 {$IFDEF LINUX}
 While Not ibservice.Eof do
 begin
  Memo.lines.add(ibservice.GetNextLine);
  Application.processmessages;
 end;
 Memo.SelStart:=length(Memo.Text);
 ASave.enabled:=True;
 ACopy.Enabled:=True;
 Isterminated:=true;
 {$ENDIF}
end;

procedure TIBProcesThread.SyncMessage;
begin
 if not assigned(form) then
  exit;
 with form do
 begin
  if assigned(form.memo) then
  begin
   Memo.lines.add(Msg);
   Memo.SelStart:=length(Memo.Text);
  end;
 end;
end;

procedure TIBProcesThread.Execute;
begin
 try
  While Not Fibservice.Eof do
  begin
   msg:=Fibservice.GetNextLine;
   Synchronize(SyncMessage);
  end;
 except
  on E:Exception do
  begin
   Msg:=E.Message;
   Synchronize(SyncMessage);
  end;
 end;
end;

{$IFDEF MSWINDOWS}
procedure TFIBPRoces.TTerminate(Sender:TObject);
begin
 isterminated:=true;
 ASave.enabled:=True;
 ACopy.Enabled:=True;
end;
{$ENDIF}

procedure TFIBProces.ACloseExecute(Sender: TObject);
begin
 CLose;
end;

procedure TFIBProces.ACopyExecute(Sender: TObject);
begin
 Memo.CopyToClipboard;
end;

procedure TFIBProces.ASaveExecute(Sender: TObject);
begin
 if savedialog1.Execute then
 begin
  Memo.Lines.SaveToFile(savedialog1.filename);
 end;
end;

procedure TFIBProces.FormClose(Sender: TObject; var Action: TCloseAction);
begin
 if Not Isterminated then
 begin
  Raise exception.create(SRpServiceRunning);
 end;
end;


end.
