{*******************************************************}
{                                                       }
{       IBAccess                                        }
{       Restore database dialog                         }
{                                                       }
{       Author:                                         }
{       Toni Martir                                     }
{       http://www.ibaccess.org                         }
{       e-mail: toni@pala.com                           }
{                                                       }
{       OpenSource license (MPL)                        }
{                                                       }
{*******************************************************}

unit ibresto;

interface

uses
  SysUtils, Classes, QForms,variants,
  Db, IBDatabase, ibmanst,
  ibman,rpibdb, IBServices,dbclient, QDialogs, QStdCtrls, QButtons,
  QControls, QGrids, QDBGrids;

type
  TFIBRestore = class(TForm)
    DBGrid1: TDBGrid;
    DataSource1: TDataSource;
    Label1: TLabel;
    BRestore: TButton;
    BCancelar: TButton;
    BSearch: TBitBtn;
    Label4: TLabel;
    ibman1: TIbManager;
    OpenDialog1: TOpenDialog;
    DMemtable: TClientdataset;
    DMemtableFile: TStringField;
    DMemtableSize: TIntegerField;
    DMemtableSizeMb: TIntegerField;
    GConf: TGroupBox;
    LAlias: TLabel;
    BCOnfig: TBitBtn;
    ComboAlias: TComboBox;
    Label6: TLabel;
    LProtocol: TLabel;
    EServer: TEdit;
    ComboProtocol: TComboBox;
    Label3: TLabel;
    Label2: TLabel;
    Label5: TLabel;
    EPassword: TEdit;
    EUsername: TEdit;
    ComboPage: TComboBox;
    GRestore: TGroupBox;
    RadioService: TRadioButton;
    RadioCommandline: TRadioButton;
    CheckService: TCheckBox;
    DFiles: TClientdataset;
    DataSource2: TDataSource;
    DBGrid2: TDBGrid;
    DFilesFile: TStringField;
    GOptions: TGroupBox;
    Checknoindex: TCheckBox;
    CheckNoShadow: TCheckBox;
    CheckNoValidity: TCheckBox;
    CheckReplace: TCheckBox;
    CheckFullSpace: TCheckBox;
    CheckReadonly: TCheckBox;
    CheckCommit: TCheckBox;
    EBuffers: TEdit;
    Label7: TLabel;
    Label8: TLabel;
    BShowops: TButton;
    BHideOps: TButton;
    IBRestore: TIBRestoreService;
    procedure FormShow(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure BRestoreClick(Sender: TObject);
    procedure BSearchClick(Sender: TObject);
    procedure ComboPageChange(Sender: TObject);
    procedure BCOnfigClick(Sender: TObject);
    procedure ComboAliasClick(Sender: TObject);
    procedure EServerChange(Sender: TObject);
    procedure DMemtableCalcFields(DataSet: TDataSet);
    procedure RadioCommandlineClick(Sender: TObject);
    procedure BShowopsClick(Sender: TObject);
    procedure BHideOpsClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
  private
    { Private declarations }
   dbconf:TRpDBIBconf;
   restored:boolean;
   username,password,server,database:string;
   protocol:TrpIBXProtocol;
  public
    { Public declarations }
  end;


procedure ShowIBRestoreDialog(var username,password,server,database:string;var protocol:TRpIBXProtocol;dbconf:TRpDBIBconf);

implementation

uses ibproces;

{$R *.xfm}

procedure TFIBRestore.FormShow(Sender: TObject);
begin
 DMemtable.close;
 DMemTable.Createdataset;
 DFiles.close;
 DFiles.createdataset;
 ComboPage.itemindex:=3;
 comboprotocol.itemindex:=0;
end;

procedure TFIBRestore.FormDestroy(Sender: TObject);
begin
 DMemTable.Close;
end;

procedure ShowIBRestoreDialog(var username,password,server,database:string;var protocol:TRpIBXProtocol;dbconf:TRpDBIBconf);
var
 dia:TFIBRestore;
begin
 dia:=TFIBRestore.Create(Application);
 try
  dia.EUsername.text:=username;
  dia.EPassword.text:=password;
  dia.EServer.text:=server;
  if length(database)>0 then
  begin
   dia.DMemtable.Append;
   dia.DMemtablefile.value:=database;
   dia.DMemtable.post;
  end;
  dia.dbconf:=dbconf;
  if assigned(dbconf) then
  begin
   dia.GConf.visible:=true;
   dia.comboalias.items.assign(dbconf.databases);
  end;
  dia.Radiocommandline.checked:=ibman.globalusecommandline;
  dia.showmodal;
  if dia.restored then
  begin
   username:=dia.Username;
   password:=dia.Password;
   protocol:=dia.Protocol;
   server:=dia.Server;
   database:=dia.database;
  end;
 finally
  dia.free;
 end;
end;

procedure TFIBRestore.BRestoreClick(Sender: TObject);
var
 astring:string;
 server:string;
 source:string;
begin
 restored:=True;
 server:=EServer.text;
 protocol:=TRpIBXPRotocol(comboprotocol.itemindex);
 username:=EUsername.text;
 password:=EPassword.text;
 DMemTable.checkbrowsemode;
 DFiles.checkbrowsemode;
 if (DMemtable.EOF AND DMemTable.BOF) then
  Raise Exception.Create(SIBEspeFiles);
 if (DFiles.EOF AND DFIles.BOF)  then
  Raise Exception.Create(SIBEspeFile);
 server:='';
 DMemtable.disablecontrols;
 try
  // Check for correct values
  DMemTable.first;
  While Not DMemTable.eof do
  begin
   if DMemTableSize.IsNull then
    break;
   DMemTable.next;
  end;
  DMemtable.next;
  if Not DMemTable.EOF then
   Raise Exception.Create(SIBSecTamany);
  if Not DMemTablesize.isnull then
  begin
   DMemTable.edit;
   try
    DMemTablesize.asvariant:=null;
    DMemTable.post;
   except
    DMemTable.cancel;
    raisE;
   end;
  end;
  DMemTable.first;
  database:=DMemTableFile.Value;
  IBRestore.DatabaseName.clear;
  IBRestore.BackupFile.clear;
  IBRestore.ServerName:=Trim(EServer.Text);
  ibman1.serverpath:=ibman.globalserverpath;
  ibman1.restorefiles.clear;
  While Not DMemtable.eof do
  begin
   if dmemtable.bof then
    ibman1.restorefiles.add(BuildIBDatabaseName((Length(Trim(EServer.text))>0),TRpIBXProtocol(ComboProtocol.ItemIndex),EServer.Text,DMemTablefile.asstring))
   else
    ibman1.restorefiles.add(DMemtableFile.asstring);
   if Not DMemTableSize.isnull then
   begin
    ibman1.restorefiles.add(DMemtableSize.asstring);
    IBRestore.DatabaseName.add(dmemtablefile.asstring
     +'='+DMemtablesize.asstring);
   end
   else
   begin
    IBRestore.DatabaseName.add(dmemtablefile.asstring);
   end;
   DMemTable.next;
  end;

  DFiles.disablecontrols;
  try
   DFiles.First;
   source:='';
   ibman1.backupfiles.clear;
   While not DFiles.eof do
   begin
    ibman1.backupfiles.add(DFilesfile.asstring);
    IBRestore.BackupFile.add(DFilesfile.asstring);
    DFiles.next;
   end;
  finally
   DFiles.enablecontrols;
  end;
  if RadioService.checked then
  begin
   IBRestore.Params.clear;
   IBRestore.params.add('user_name='+EUsername.text);
   IBRestore.params.add('password='+EPassword.text);
   if Length(IBRestore.ServerName)=0 then
   begin
    IBRestore.protocol:=Local;
   end
   else
   begin
    IBRestore.protocol:=TProtocol(ComboProtocol.itemindex);
   end;
   IBRestore.PageSize:=StrToInt(ComboPage.Text);
   IBRestore.options:=[CreateNewDB];
   if CheckNoIndex.checked then
    IBRestore.options:=IBRestore.options+[Deactivateindexes];
   if CheckNoShadow.checked then
    IBRestore.options:=IBRestore.options+[NoShadow];
   if CheckNoValidity.checked then
    IBRestore.options:=IBRestore.options+[NoValidityCheck];
   if CheckReplace.checked then
    IBRestore.options:=IBRestore.options+[Replace];
   if CheckCommit.checked then
    IBRestore.options:=IBRestore.options+[OneRelationAtATime];
   if CheckFullspace.checked then
    IBRestore.options:=IBRestore.options+[UseAllSpace];
   IBRestore.BufferSize:=StrToInt(EBuffers.Text);
   IBRestore.Attach;
   try
    IBRestore.ServiceStart;
    ibproces.COntrolIBService(IBRestore);
   finally
    try
     IBRestore.Detach;
    except
    end;
   end;
  end
  else
  begin
   ibman1.PageSize:=tpagesizetype(Combopage.itemindex);
   ibman1.username:=EUsername.text;
   ibman1.Password:=EPassword.text;
   Ibman1.Filename:=astring;
   IbMan1.RestoreOptions:=[];
   if CheckReplace.checked then
    IbMan1.RestoreOptions:=IbMan1.RestoreOptions+[rReplaceexisting];
   if CheckReadOnly.checked then
    IbMan1.RestoreOptions:=IbMan1.RestoreOptions+[rReadOnly];
   if CheckNoShadow.checked then
    IbMan1.RestoreOptions:=IbMan1.RestoreOptions+[rNoShadows];
   if CheckNoIndex.checked then
    IbMan1.RestoreOptions:=IbMan1.RestoreOptions+[rDeactivateindexes];
   if CheckCommit.checked then
    IbMan1.RestoreOptions:=IbMan1.RestoreOptions+[rCommiteachtable];
   if CheckNoValidity.checked then
    IbMan1.RestoreOptions:=IbMan1.RestoreOptions+[rNoValidate];
   if CheckFullSpace.checked then
    IbMan1.RestoreOptions:=IbMan1.RestoreOptions+[rUseAllSpace];
   ibman1.RestoreBuffers:=StrToInt(EBuffers.text);
   if Checkservice.checked then
   begin
    if length(Trim(Eserver.Text))>0 then
    begin
     case Comboprotocol.itemindex of
      0:
       begin
        ibman1.service:=Trim(EServer.text)+':'+'service_mgr';
       end;
      1:
       begin
        ibman1.service:='\\'+Trim(EServer.text)+'\'+'service_mgr';
       end;
      else
      begin
       ibman1.service:=Trim(EServer.text)+'@'+'service_mgr';
      end;
     end;
    end
    else
    begin
     ibman1.service:='service_mgr';
    end;
   end;
   ibman1.Restore;
  end;
 finally
  DMemTable.enablecontrols;
 end;
end;

procedure TFIBRestore.BSearchClick(Sender: TObject);
begin
 if Opendialog1.execute then
 begin
  if Not (DFiles.state in dseditmodes) then
  begin
   if (DFiles.Eof and DFiles.BOF) then
    DFiles.append
   else
    dfiles.edit;
   DFilesFile.AsString:=Opendialog1.filename;
  end;
 end;
end;


procedure TFIBRestore.ComboPageChange(Sender: TObject);
begin
 DMemTable.disablecontrols;
 try
  DMemTable.first;
  While Not DMemtable.eof do
  begin
   DMemtable.edit;
   try
    DMemTableSize.asvariant:=DMemTableSize.asvariant;
    DMemTable.post;
   except
    DMemtable.cancel;
    raise;
   end;
   DMemtable.Next;
  end;
 finally
  DMemTable.enablecontrols;
 end;
end;

procedure TFIBRestore.BCOnfigClick(Sender: TObject);
begin
 dbconf.ShowConfig(True,True,true);
 comboalias.Items.Assign(dbconf.databases);
 comboalias.itemindex:=-1;
end;

procedure TFIBRestore.ComboAliasClick(Sender: TObject);
var
 dbinfo:TRpDBIBinfo;
begin
 if Comboalias.itemindex<0 then
  exit;
 dbinfo:=dbconf.databases.items[comboalias.itemindex];
 ComboProtocol.Itemindex:=integer(dbinfo.protocol);
 EServer.text:=dbinfo.Server;
 EserverChange(Self);
 if (DMemtable.state in dseditmodes) then
  DMemtable.cancel;
 DMemTable.close;
 DMemTable.createdataset;
 DMemTable.insert;
 DMemtableFile.Value:=dbinfo.Filename;
 EUsername.text:=dbinfo.username;
 EPassword.text:=dbinfo.password;
end;

procedure TFIBRestore.EServerChange(Sender: TObject);
begin
 if Length(Trim(Eserver.Text))>0 then
 begin
  LProtocol.Visible:=True;
  ComboProtocol.Visible:=True;
 end
 else
 begin
  LProtocol.Visible:=False;
  ComboProtocol.Visible:=False;
 end;
end;

procedure TFIBRestore.DMemtableCalcFields(DataSet: TDataSet);
begin
  if DMemTableSize.isnull then
  DMemTableSizeMB.asvariant:=null
 else
  DMemTableSizeMb.asinteger:=DMemTableSize.AsInteger*(StrToInt(ComboPage.items[ComboPage.itemindex]) div 1024) div 1024;
end;

procedure TFIBRestore.RadioCommandlineClick(Sender: TObject);
begin
 if RadioCommandline.Checked then
 begin
  CheckService.Enabled:=True;
  Checkreadonly.enabled:=True;
 end
 else
 begin
  CheckService.Enabled:=False;
  Checkreadonly.checked:=False;
  Checkreadonly.enabled:=False;
 end;
end;

procedure TFIBRestore.BShowopsClick(Sender: TObject);
begin
 Width:=GOptions.Left+Goptions.Width+5;
 BhideOps.Visible:=True;
 BShowOps.Visible:=False;
 GOptions.Visible:=True;
end;

procedure TFIBRestore.BHideOpsClick(Sender: TObject);
begin
 Width:=BShowops.Left+BShowOps.Width+5;
 BhideOps.Visible:=False;
 BShowOps.Visible:=True;
 GOptions.Visible:=False;
end;

procedure TFIBRestore.FormActivate(Sender: TObject);
begin
 Width:=BShowops.Left+BShowOps.Width+5;
end;

end.
