{*******************************************************}
{                                                       }
{       IBAccess                                        }
{       Persistant Class                                }
{                                                       }
{       Author:                                         }
{       Kevin Curtis                                    }
{       e-mail: kevin@curtissoftware.com                }
{                                                       }
{       OpenSource license (MPL)                        }
{                                                       }
{*******************************************************}

unit PersistentForm;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes,
  Graphics, Controls, Forms, Dialogs, IniFiles, Rpshfolder, rpstdutils;

type
  TPersistentForm = class(TForm)
  private
    { Private declarations }
    IniFile: TIniFile;
    WinState: TWindowState;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWnd; override;
  public
    { Public declarations }
    procedure Save;
    destructor Destroy; override;
  end;

implementation

procedure TPersistentForm.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Inifile := TInifile.Create(Obtainininameuserconfig('Techni-web', 'IbAccess', 'Forms'));
  if IniFile <> nil then
  begin
    with IniFile do
    begin
      try
        if SectionExists(Self.ClassName) then
        begin
          if ValueExists(Self.ClassName, 'X') then
          begin
            with Params do
            begin
              if ValueExists(Self.ClassName, 'Position') then
                Self.Position := TPosition(ReadInteger(Self.ClassName, 'Position', Integer(poDefault)));
              if ValueExists(Self.ClassName, 'State') then
              	WinState := TWindowState(ReadInteger(Self.ClassName, 'State', Integer(TForm(Self).WindowState)));
              if ValueExists(Self.ClassName, 'X') then
              	X := ReadInteger(Self.ClassName, 'X', 0);
              if ValueExists(Self.ClassName, 'Y') then
                Y := ReadInteger(Self.ClassName, 'Y', 0);
              if ValueExists(Self.ClassName, 'Width') then
                Width := ReadInteger(Self.ClassName, 'Width', TForm(Self).Width);
              if ValueExists(Self.ClassName, 'Height') then
                Height := ReadInteger(Self.ClassName, 'Height', TForm(Self).Height);
            end;
          end
          else // Set this to your desired default position.
            Position := poScreenCenter;
        end;
      finally
        IniFile.Free;
      end;
    end;
  end;
end;


destructor TPersistentForm.Destroy;
begin
  try
    Save;
  finally
    inherited Destroy;
  end;
end;

procedure TPersistentForm.Save;
begin
  IniFile := TInifile.Create(Obtainininameuserconfig('Techni-web', 'IbAccess', 'Forms'));
  try
    with IniFile do
    begin
      WriteInteger(Self.ClassName, 'X', Left);
      WriteInteger(Self.ClassName, 'Y', Top);
      WriteInteger(Self.ClassName, 'Width', Width);
      WriteInteger(Self.ClassName, 'Height', Height);
      WriteInteger(Self.ClassName, 'Position', Integer(Self.Position));
      WriteInteger(Self.ClassName, 'State', Integer(WindowState));
    end;
  finally
    IniFile.Free;
  end;
end;

procedure TPersistentForm.CreateWnd;
begin
  inherited CreateWnd;
	{--------------------------------------------------------------
	  WindowState gets overriden and set to wsNormal in the
	  overriden CreateWnd if the FormStyple is fsMDIChild.
	  Therefore, we override it herewith the value we stored
	  when or CreateParams procedure, which was previously called.
	--------------------------------------------------------------}
  Self.WindowState := WinState;	// WinState is initially assigned in Self.CreateParams()
end;
end.

