{*******************************************************}
{                                                       }
{       IBAccess                                        }
{       New trigger and alter trigger form              }
{                                                       }
{       Author:                                         }
{       Toni Martir                                     }
{       http://www.ibaccess.org                         }
{       e-mail: toni@pala.com                           }
{                                                       }
{       OpenSource license (MPL)                        }
{                                                       }
{*******************************************************}

unit newtrigger;

interface

uses Classes, QForms,SysUtils,QDialogs,
  QMask, IBSQL, IBDatabase,
  ibacconst,QMenus, QTypes, QActnList, QStdCtrls, QComCtrls,
  QControls, QExtCtrls;

type
  TFnewTrigger = class(TForm)
    Panel1: TPanel;
    Label1: TLabel;
    ETable: TEdit;
    Label2: TLabel;
    ComboType: TComboBox;
    Label3: TLabel;
    EPosition: TMaskEdit;
    Label4: TLabel;
    ETriggername: TEdit;
    ToolBar1: TToolBar;
    ActionList1: TActionList;
    MainMenu1: TMainMenu;
    MEdit: TMenuItem;
    ACompile: TAction;
    Compile1: TMenuItem;
    ToolButton1: TToolButton;
    Tran1: TIBTransaction;
    IBSQL1: TIBSQL;
    CheckActive: TCheckBox;
    TrigEdit: TMemo;
    procedure TrigEditChange(Sender: TObject);
    procedure ACompileExecute(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ComboTypeClick(Sender: TObject);
  private
    { Private declarations }
    fforcreate:boolean;
    procedure SetForCreate(Value:boolean);
  public
    { Public declarations }
    base:TIBDatabase;
    dbform:TForm;
    isdestroying:boolean;
    property forcreate:boolean read fforcreate write setforcreate default false;
  end;


implementation

uses mainform,databaseform;

{$R *.xfm}

procedure TFNewTrigger.SetForCreate(Value:boolean);
begin
 FForcreate:=Value;
 if FForcreate then
 begin
  ETriggername.readonly:=false;
//  Etriggername.color:=clwindow;
  Etriggername.text:='NEWTRIGGER';
 end
 else
  Activecontrol:=TrigEdit;
end;

procedure TFnewTrigger.TrigEditChange(Sender: TObject);
begin
 ACompile.enabled:=True;
end;

procedure TFnewTrigger.ACompileExecute(Sender: TObject);
var
 doalter:boolean;
 position:integer;
 dform:TFDatabase;
 index:integer;
 trigname,taname:string;
 astring:string;
begin
 if Length(Trim(Etriggername.text))<1 then
 begin
  ETriggername.SetFocus;
  Raise Exception.Create(SRpTriggermusthavename);
 end;
 position:=StrToInt(EPosition.Text);
 doalter:=false;
 Tran1.defaultdatabase:=base;
 IBSQL1.database:=base;
 Tran1.Starttransaction;
 try
  // Look if exists
  IBSQL1.SQL.Text:='SELECT RDB$TRIGGER_NAME'+
   ' FROM RDB$TRIGGERS '+
   ' WHERE RDB$TRIGGER_NAME='+QuotedSTr(ETriggername.text);
  IBSQL1.Execquery;
  try
   If Not IBSQL1.EOF then
   begin
    if fforcreate then
     if mrYEs<>Messagedlg(SRpTriggerExists,mtWarning,[mbYes,mbNo,mbcancel],0) then
      Raise EAbort.create('');
    doalter:=true;
   end;
  finally
   IBSQL1.close;
  end;
  trigname:=ETriggername.text;
  taname:=ETable.text;
  if base.SQLDialect=3 then
  begin
   trigname:='"'+trigname+'"';
   taname:='"'+taname+'"';
  end;
  if doalter then
   astring:='ALTER TRIGGER '+trigname
  else
   astring:='CREATE TRIGGER '+trigname+
    ' FOR  '+taname;
  if checkactive.checked then
   astring:=astring+' ACTIVE '
  else
   astring:=astring+' INACTIVE ';
  astring:=astring+ComboType.text;
  astring:=astring+' POSITION '+IntToStr(Position)+' ';
  astring:=astring+TrigEdit.Text;
  IBSQL1.SQL.TExt:=astring;
  IBSQL1.ExecQuery;
  if fforcreate then
  begin
   dform:=TFDatabase(dbform);
   index:=dform.tableforms.indexofobject(self);
   if index>=0 then
   begin
    dform.triggerforms.strings[index]:=ETriggername.text;
    caption:=SRpTrigger+':'+Etriggername.text+' - '+base.databasename;
   end;
  end;
  fforcreate:=false;
  ETriggername.ReadOnly:=True;
//  ETriggername.color:=clinfobk;
  Acompile.enabled:=false;
  Tran1.commit;
 except
  Tran1.rollback;
  raise;
 end;
end;

procedure TFnewTrigger.FormCreate(Sender: TObject);
begin
 TrigEdit.text:='AS '+#13+'BEGIN'+#13+#13+'END';
 ComboType.itemindex:=0;
 height:=350;
end;

procedure TFnewTrigger.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
var
 res:Tmodalresult;
begin
 if ACompile.Enabled then
 begin
  res:=MessageDlg(Format(SrpSaveTrigger,[ETriggername.text]),mtWarning,[mbYes,mbNo,mbcancel],0);
  if res=mrcancel then
   canclose:=False
  else
   if res=mrYes then
    ACompile.execute;
 end;
end;

procedure TFnewTrigger.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
 action:=cafree;
 isdestroying:=true;
 if fmain.DWindows.Locate('WINDOW',integer(Self),[]) then
  fmain.DWindows.delete;
end;

procedure TFnewTrigger.ComboTypeClick(Sender: TObject);
begin
 Acompile.enabled:=true;
end;
 end.
