{*******************************************************}
{                                                       }
{       IBAccess                                        }
{       Open database dialog                            }
{                                                       }
{       Author:                                         }
{       Toni Martir                                     }
{       http://www.ibaccess.org                         }
{       e-mail: toni@pala.com                           }
{                                                       }
{       OpenSource license (MPL)                        }
{                                                       }
{*******************************************************}

unit opendatabase;

interface

uses
  SysUtils, Classes, QGraphics, QForms, 
  rpibdb, ibdatabase,Db, QDialogs, QButtons, QStdCtrls, QControls;

type
  TFOpenDatabase = class(TForm)
    BOK: TButton;
    BCancel: TButton;
    Label4: TLabel;
    ComboChar: TComboBox;
    OpenDialog1: TOpenDialog;
    Erole: TEdit;
    Label5: TLabel;
    Label3: TLabel;
    Label2: TLabel;
    Label1: TLabel;
    Label6: TLabel;
    LAlias: TLabel;
    LProtocol: TLabel;
    EPassword: TEdit;
    EUsername: TEdit;
    BSearch: TBitBtn;
    EBase: TEdit;
    BCOnfig: TBitBtn;
    ComboAlias: TComboBox;
    EServer: TEdit;
    ComboProtocol: TComboBox;
    Label10: TLabel;
    ComboVersion: TComboBox;
    procedure BExaminarClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure BCOnfigClick(Sender: TObject);
    procedure ComboAliasClick(Sender: TObject);
    procedure EBaseChange(Sender: TObject);
    procedure BSearchClick(Sender: TObject);
    procedure EServerChange(Sender: TObject);
    procedure BOKClick(Sender: TObject);
  private
    { Private declarations }
    dbconf:TRpDBIBConf;
    dbase:TIBDatabase;
    protocol:TrpIBXProtocol;
    procedure DoOpenDatabase;
  public
    { Public declarations }
  end;

function OpenIbDatabase(base:TIBDatabase;var server,filename,username,password,role,chartype:string;var protocol:TrpIBXProtocol;dialect:integer;dbconf:TRpDBIBconf):boolean;

implementation


{$R *.xfm}

function OpenIbDatabase(base:TIBDatabase;var server,filename,username,password,role,chartype:string;var protocol:TrpIBXProtocol;dialect:integer;dbconf:TRpDBIBconf):boolean;
var
 dia:TFOpenDatabase;
begin
 Result:=false;
 // Mostrem un dialeg d'obrir
 dia:=TFOpenDatabase.create(application);
 try
  dia.EUsername.text:=username;
  dia.Epassword.text:=password;
  dia.Erole.text:=role;
  dia.dbase:=base;
  dia.dbconf:=dbconf;
  if dialect=1 then
   dia.Comboversion.itemindex:=0
  else
   dia.Comboversion.itemindex:=1;
  dia.showmodal;
  if base.connected then
  begin
   username:=dia.EUsername.text;
   password:=dia.EPassword.text;
   server:=dia.EServer.text;
   filename:=dia.Ebase.text;
   role:=dia.eRole.text;
   protocol:=dia.protocol;
   chartype:=dia.Combochar.Text;
   Result:=True;
  end;
 finally
  dia.free;
 end;
end;


procedure TFOpenDatabase.BExaminarClick(Sender: TObject);
begin
 if Opendialog1.execute then
  EBase.text:=Opendialog1.filename;
end;

procedure TFOpenDatabase.FormShow(Sender: TObject);
begin
 if Assigned(dbconf) then
 begin
  LAlias.visible:=True;
  ComboAlias.visible:=true;
  BConfig.visible:=True;
  comboalias.Items.Assign(dbconf.databases);
 end;
 if combochar.itemindex<0 then
  ComboChar.itemindex:=Combochar.items.indexof('WIN1252');
 ComboProtocol.Itemindex:=0;
end;

procedure TFOpenDatabase.BCOnfigClick(Sender: TObject);
begin
 dbconf.ShowConfig(True,True,true);
 dbconf.LoadAliases;
 comboalias.Items.Assign(dbconf.databases);
 comboalias.itemindex:=-1;
end;

procedure TFOpenDatabase.ComboAliasClick(Sender: TObject);
var
 dbinfo:TRpDBIBinfo;
begin
 if Comboalias.itemindex<0 then
  exit;
 dbinfo:=dbconf.databases.items[comboalias.itemindex];
 EServer.text:=dbinfo.Server;
 EserverChange(Self);
 EBase.text:=dbinfo.Filename;
 EUsername.text:=dbinfo.username;
 EPassword.text:=dbinfo.password;
 ERole.text:=dbinfo.Role;
 ComboChar.itemindex:=Combochar.items.indexof(dbinfo.CharacterSet);
 ComboProtocol.Itemindex:=integer(dbinfo.protocol);
 case dbinfo.SQLDIalect of
  1:
   ComboVersion.itemindex:=0;
  else
   ComboVersion.itemindex:=1;
 end;
end;

procedure TFOpenDatabase.EBaseChange(Sender: TObject);
begin
 Comboalias.itemindex:=0;
end;

procedure TFOpenDatabase.BSearchClick(Sender: TObject);
begin
 if opendialog1.execute then
  EBase.text:=opendialog1.filename;
end;

procedure TFOpenDatabase.EServerChange(Sender: TObject);
begin
 if Length(Trim(Eserver.Text))>0 then
 begin
  LProtocol.Visible:=True;
  ComboProtocol.Visible:=True;
 end
 else
 begin
  LProtocol.Visible:=False;
  ComboProtocol.Visible:=False;
 end;
 if Length(trim(EServer.text))>0 then
  BSearch.visible:=false
 else
  BSearch.visible:=true;
end;

procedure TFOpenDatabase.DoOpenDatabase;
begin
 // Opens the database
 with dbase do
 begin
  connected:=false;
  params.clear;
  params.add('user_name='+Eusername.text);
  params.add('password='+Epassword.Text);
  if Length(Trim(Erole.text))>0 then
   params.add('sql_role_name='+Erole.text);
  params.add('lc_ctype='+ComboChar.text);
  protocol:=TRpIBXProtocol(ComboProtocol.Itemindex);
  if comboversion.itemindex=0 then
   sqldialect:=1
  else
   sqldialect:=3;
  databasename:=BuildIBDatabaseName((Length(Trim(EServer.text))>0),TRpIBXProtocol(ComboProtocol.ItemIndex),EServer.Text,EBase.text);
  connected:=true;
 end;
end;

procedure TFOpenDatabase.BOKClick(Sender: TObject);
begin
 DoOpenDatabase;
 Close;
end;

end.
