{*******************************************************}
{                                                       }
{       IBAccess                                        }
{       Access control list to database objects         }
{                                                       }
{       Author:                                         }
{       Toni Martir                                     }
{       http://www.ibaccess.org                         }
{       e-mail: toni@pala.com                           }
{                                                       }
{       OpenSource license (MPL)                        }
{                                                       }
{*******************************************************}

unit roleaccess;

interface

uses
  SysUtils, Classes, QGraphics, QForms,
  Db, IBSQL, IBDatabase,
  ibacconst, rpibdb, DBClient, QActnList, QMenus, QTypes, QGrids, QDBGrids,
  QStdCtrls, QButtons, QControls, QExtCtrls;

type
  TFRoleAccess = class(TForm)
    Panel1: TPanel;
    Label1: TLabel;
    ComboShow: TComboBox;
    ComboRole: TComboBox;
    BNewfield: TBitBtn;
    BRemovefield: TBitBtn;
    GridPriv: TDBGrid;
    Trans: TIBTransaction;
    IBSQL1: TIBSQL;
    DPrivileges: TClientdataset;
    DPrivilegesUSER: TStringField;
    DPrivilegesPRIVILEGE: TStringField;
    DPrivilegesGRANTOR: TStringField;
    DPrivilegesGRANT_OPTION: TIntegerField;
    DPrivilegesRELATION_NAME: TStringField;
    DPrivilegesFIELD_NAME: TStringField;
    DPrivilegesUSER_TYPE: TIntegerField;
    DPrivilegesOBJECT_TYPE: TIntegerField;
    DataSource1: TDataSource;
    DPrivilegesTYPE_NAME: TStringField;
    DPrivilegesPRIVILEGENAME: TStringField;
    DPrivilegesGRANTADMIN: TStringField;
    DPrivilegesUSER_TYPE_NAME: TStringField;
    MainMenu1: TMainMenu;
    MEdit: TMenuItem;
    Compile1: TMenuItem;
    ActionList1: TActionList;
    AAutorefresh: TAction;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure BNewfieldClick(Sender: TObject);
    procedure ComboShowClick(Sender: TObject);
    procedure ComboRoleClick(Sender: TObject);
    procedure ComboRoleDropDown(Sender: TObject);
    procedure ComboRoleChange(Sender: TObject);
    procedure ComboRoleExit(Sender: TObject);
    procedure BRemovefieldClick(Sender: TObject);
    procedure AAutorefreshExecute(Sender: TObject);
  private
    { Private declarations }
    fbase:TIBDatabase;
    validlist:boolean;
    rolechanged,didclick:boolean;
    procedure SetBase(Value:TIBDatabase);
    procedure BefPost(IBSQL1:TIBSQL;sender:TDataset);
  public
    { Public declarations }
    isdestroying:boolean;
    user,password,server:string;
    protocol:TRpIBXProtocol;
    procedure ReadAll;
    procedure Updatedata;
    property base:TIBDatabase read fbase write Setbase;
  end;

var
  FRoleAccess: TFRoleAccess;


function ObjectTypeToString(otype:integer):string;
function PrivilegeCharToString(priv:string):string;


implementation

uses ibutil2, ibacutils, newgrant, mainform;

{$R *.xfm}

function ObjectTypeToString(otype:integer):string;
begin
 case otype of
  0:
   Result:='TABLE';
  1:
   Result:='VIEW';
  2:
   Result:='TRIGGER';
  3:
   Result:='COMPUTED FIELD';
  4:
   Result:='VALIDATION';
  5:
   Result:='PROCEDURE';
  6:
   Result:='EXPRESSION_INDEX';
  7:
   Result:='EXCEPTION';
  8:
   Result:='USER';
  9:
   Result:='FIELD';
  10:
   Result:='INDEX';
  13:
   Result:='ROLE';
  else
   Result:='NONE';
 end;
end;

function PrivilegeCharToString(priv:string):string;
begin
 if Length(priv)<1 then
 begin
  Result:='NONE';
 end;
 case priv[1] of
  'A':
   Result:='ALL';
  'D':
   Result:='DELETE';
  'I':
   Result:='INSERT';
  'M':
   Result:='MEMBER';
  'R':
   Result:='REFERENCES';
  'S':
   Result:='SELECT';
  'U':
   Result:='UPDATE';
  'X':
   Result:='EXECUTE';
  else
   Result:='NONE';
 end;
end;

procedure TFRoleAccess.Setbase(value: TIBDatabase);
begin
 fbase:=value;
 Trans.defaultdatabase:=fbase;
 IBSQL1.database:=fbase;
 if value=nil then
  exit;
end;

procedure TFRoleAccess.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
 action:=cafree;
 isdestroying:=True;
 if fmain.DWindows.Locate('WINDOW',integer(Self),[]) then
  fmain.DWindows.delete;
end;

procedure TFRoleAccess.ReadAll;
begin
 ComboShow.Itemindex:=0;
 ComboRole.Text:='';
 Updatedata;
end;

procedure TFRoleAccess.BefPost(IBSQL1:TIBSQL;sender:TDataset);
var
 i:integer;
begin
 for i:=0 to ibsql1.current.count-1 do
 begin
  if sender.Fields[i].DataType in [ftstring,ftMemo] then
   sender.fields[i].Value:=ibsql1.fields[i].AsString
  else
   sender.fields[i].Value:=ibsql1.fields[i].Value;
 end;
 // Assign other fields
 DPrivilegesTYPE_NAME.Value:=ObjectTypeToString(IBSQL1.FieldByName('RDB$OBJECT_TYPE').Asinteger);
 DPrivilegesUSER_TYPE_NAME.Value:=ObjectTypeToString(IBSQL1.FieldByName('RDB$USER_TYPE').Asinteger);
 DPrivilegesPRIVILEGENAME.Value:=PrivilegeCharToString(IBSQL1.FieldByName('RDB$PRIVILEGE').Asstring);
 DPrivilegesGRANTADMIN.AsString:=SRpNo;
 if (NOT IBSQL1.FieldByName('RDB$GRANT_OPTION').IsNull) then
 begin
  if IBSQL1.FieldByName('RDB$GRANT_OPTION').AsInteger>0 then
   DPrivilegesGRANTADMIN.AsString:=SRpYes;
 end;
end;

procedure TFRoleAccess.Updatedata;
var
 sql:string;
 cond:string;
begin
 validlist:=false;
 sql:='SELECT U.RDB$USER,U.RDB$GRANTOR,U.RDB$PRIVILEGE,'+
  'U.RDB$GRANT_OPTION,U.RDB$RELATION_NAME,U.RDB$FIELD_NAME,'+
  'U.RDB$USER_TYPE,U.RDB$OBJECT_TYPE '+
  'FROM RDB$USER_PRIVILEGES U';
 cond:=' WHERE (1=1) ';
 case ComboShow.itemindex of
  1:
   cond:=cond+' AND (U.RDB$USER_TYPE=13) ';
  2:
   cond:=cond+' AND (U.RDB$USER_TYPE=8) ';
  3:
   cond:=cond+' AND (U.RDB$USER_TYPE=0) ';
  4:
   cond:=cond+' AND (U.RDB$USER_TYPE=1) ';
  5:
   cond:=cond+' AND (U.RDB$USER_TYPE=5) ';
  6:
   cond:=cond+' AND (U.RDB$USER_TYPE=2) ';
 end;
 if Length(Trim(ComboRole.text))>0 then
 begin
  cond:=cond+' AND (U.RDB$USER='+Quotedstr(ComboRole.text)+')';
 end;
 sql:=sql+cond;
 sql:=sql+' ORDER BY U.RDB$USER_TYPE,U.RDB$USER,U.RDB$OBJECT_TYPE,U.RDB$RELATION_NAME';
 dprivileges.close;
 dprivileges.createdataset;
 dprivileges.disablecontrols;
 try
  FillDataset(IBSQL1,Trans,SQL,DPrivileges,BefPost);
  dprivileges.first;
 finally
  dprivileges.enablecontrols;
 end;
end;

procedure TFRoleAccess.BNewfieldClick(Sender: TObject);
begin
 if ibnewgrant(base,user,password,server,protocol) then
  if aautorefresh.checked then
   Updatedata;
end;

procedure TFRoleAccess.ComboShowClick(Sender: TObject);
begin
 ComboRole.text:='';
 Updatedata;
end;

procedure TFRoleAccess.ComboRoleClick(Sender: TObject);
begin
 //
 rolechanged:=false;
 didclick:=true;
 Updatedata;
end;

procedure TFRoleAccess.ComboRoleDropDown(Sender: TObject);
var
 astring:string;
begin
 if validlist then
  exit;
 validlist:=true;
 comborole.items.clear;
 case ComboShow.ItemIndex of
  0:
   begin
    FillWithUsers(user,password,server,protocol,comborole.items);
    astring:='SELECT RDB$ROLE_NAME FROM RDB$ROLES '+
     ' UNION '+
     'SELECT RDB$RELATION_NAME FROM RDB$RELATIONS'+
     ' WHERE ((RDB$SYSTEM_FLAG IS NULL) OR (RDB$SYSTEM_FLAG=0)) '+
     ' UNION '+
     'SELECT RDB$PROCEDURE_NAME FROM RDB$PROCEDURES'+
     ' WHERE ((RDB$SYSTEM_FLAG IS NULL) OR (RDB$SYSTEM_FLAG=0)) '+
     ' UNION '+
     'SELECT RDB$TRIGGER_NAME FROM RDB$TRIGGERS'+
     ' WHERE ((RDB$SYSTEM_FLAG IS NULL) OR (RDB$SYSTEM_FLAG=0)) '+
     ' ORDER BY 1';
   end;
  1:
   begin
    astring:='SELECT RDB$ROLE_NAME FROM RDB$ROLES ORDER BY 1';
   end;
  2:
   begin
    astring:='';
    FillWithUsers(user,password,server,protocol,comborole.items);
   end;
  3:
   begin
    astring:='SELECT RDB$RELATION_NAME FROM RDB$RELATIONS'+
     ' WHERE (RDB$VIEW_SOURCE IS NULL) AND '+
     '((RDB$SYSTEM_FLAG IS NULL) OR (RDB$SYSTEM_FLAG=0))'+
     ' ORDER BY 1';
   end;
  4:
   begin
    astring:='SELECT RDB$RELATION_NAME FROM RDB$RELATIONS'+
     ' WHERE (NOT (RDB$VIEW_SOURCE IS NULL)) AND '+
     '((RDB$SYSTEM_FLAG IS NULL) OR (RDB$SYSTEM_FLAG=0))'+
     ' ORDER BY 1';
   end;
  5:
   begin
    astring:='SELECT RDB$PROCEDURE_NAME FROM RDB$PROCEDURES'+
     ' WHERE '+
     '((RDB$SYSTEM_FLAG IS NULL) OR (RDB$SYSTEM_FLAG=0))'+
     ' ORDER BY 1';
   end;
  6:
   begin
    astring:='SELECT RDB$TRIGGER_NAME FROM RDB$TRIGGERS'+
     ' WHERE '+
     '((RDB$SYSTEM_FLAG IS NULL) OR (RDB$SYSTEM_FLAG=0))'+
     ' ORDER BY 1';
   end;
 end;
 if astring='' then
  exit;
 Trans.starttransaction;
 try
  IBSQL1.sql.text:=astring;
  IBSQL1.Execquery;
  While NOt IBSQL1.EOF do
  begin
   Comborole.items.Add(IBSQl1.fields[0].AsString);
   IBSQL1.Next;
  end;
 finally
  Trans.rollback;
 end;
end;

procedure TFRoleAccess.ComboRoleChange(Sender: TObject);
begin
 if didclick then
  didclick:=false
 else
  rolechanged:=true;
end;

procedure TFRoleAccess.ComboRoleExit(Sender: TObject);
begin
 if rolechanged then
 begin
  rolechanged:=false;
  Updatedata;
 end;
end;

procedure TFRoleAccess.BRemovefieldClick(Sender: TObject);
var
 astring:string;
 adeli:String;
begin
 adeli:='';
 if base.sqldialect=3 then
  adeli:='"';
 if (DPrivileges.EOF AND DPrivileges.EOF) then
  exit;
 astring:='REVOKE ';
 case DPrivilegesOBJECT_TYPE.AsInteger of
  0,1:
   begin
    astring:=astring+DPrivilegesPRIVILEGENAME.AsString;
    if Not DPrivilegesFIELD_NAME.isnull then
    begin
     if Length(Trim(DPrivilegesFIELD_NAME.AsString))>0 then
      astring:=astring+' ('+adeli+DPrivilegesFIELD_NAME.AsString+adeli+')';
    end;
    astring:=astring+' ON '+adeli+DPrivilegesRELATION_NAME.AsString+adeli;
   end;
  5:
   astring:=astring+' EXECUTE ON PROCEDURE '+adeli+DPrivilegesRELATION_NAME.asstring+adeli;
  13:
   astring:=astring+adeli+DPrivilegesRELATION_NAME.asstring+adeli;
 end;
 astring:=astring+' FROM ';
 case DPrivilegesUSER_TYPE.AsInteger of
  1:
   astring:=astring+' VIEW '+adeli+DPrivilegesUSER.asstring+adeli;
  2:
   astring:=astring+' TRIGGER '+adeli+DPrivilegesUSER.asstring+adeli;
  5:
   astring:=astring+' PROCEDURE '+adeli+DPrivilegesUSER.asstring+adeli;
  8,13:
   astring:=astring+adeli+DPrivilegesUSER.asstring+adeli;
 end;
 Trans.starttransaction;
 try
  IBSQL1.SQL.text:=astring;
  IBSQL1.Execquery;
  Trans.commit;
 except
  Trans.rollback;
  raise;
 end;
 if AAutorefresh.checked then
  Updatedata;
end;

procedure TFRoleAccess.AAutorefreshExecute(Sender: TObject);
begin
 AAutorefresh.checked:=Not AAutorefresh.checked;
end;

end.
