{*******************************************************}
{                                                       }
{       IBAccess                                        }
{       Server properties dialog                        }
{                                                       }
{       Author:                                         }
{       Toni Martir                                     }
{       http://www.ibaccess.org                         }
{       e-mail: toni@pala.com                           }
{                                                       }
{       OpenSource license (MPL)                        }
{                                                       }
{*******************************************************}

unit serverprops;

interface

uses
  SysUtils, Classes, QForms, QDialogs,
  diaser, rpibdb, IBServices,ibheader, QStdCtrls, QButtons, QControls;

type
  TFServerProps = class(TFService)
    IBProp: TIBServerProperties;
    GProps: TGroupBox;
    Label4: TLabel;
    LVersion: TLabel;
    LImplem: TLabel;
    Label7: TLabel;
    Label5: TLabel;
    LActive: TLabel;
    Label8: TLabel;
    LDatabases: TLabel;
    Memodb: TMemo;
    MemoConfig: TMemo;
    Label9: TLabel;
    Label10: TLabel;
    ERoot: TEdit;
    Label11: TLabel;
    ELock: TEdit;
    Label12: TLabel;
    EMessage: TEdit;
    Label13: TLabel;
    ESec: TEdit;
    Label14: TLabel;
    IBLog: TIBLogService;
    BLog: TButton;
    procedure BOKClick(Sender: TObject);
    procedure BLogClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

procedure ShowServerIBProps(var username,password,server,database:string;var protocol:TRpIBXProtocol;dbconf:TRpDBIBconf);

implementation

uses ibproces;

{$R *.xfm}

procedure ShowServerIBProps(var username,password,server,database:string;var protocol:TRpIBXProtocol;dbconf:TRpDBIBconf);
var
 dia:TFServerProps;
begin
 dia:=TFServerprops.create(Application);
 try
  dia.Eusername.text:=username;
  dia.EPassword.text:=password;
  dia.EServer.text:=server;
  dia.dbconf:=dbconf;
  dia.EServerChange(dia);
  dia.comboprotocol.itemindex:=integer(protocol);
  dia.showmodal;
  if dia.executed then
  begin
   username:=dia.USername;
   password:=dia.Password;
   server:=dia.Server;
   protocol:=dia.protocol;
   database:=dia.database;
  end;
 finally
  dia.free;
 end;
end;

procedure TFServerProps.BOKClick(Sender: TObject);
var
 i:integer;
 astring:string;
begin
  inherited;
 IBProp.Params.clear;
 IBProp.params.add('user_name='+EUsername.text);
 IBProp.params.add('password='+EPassword.text);
 IBProp.ServerName:=Trim(EServer.Text);
 if Length(IBProp.ServerName)=0 then
 begin
  IBProp.protocol:=Local;
 end
 else
 begin
  IBProp.protocol:=TProtocol(ComboProtocol.itemindex);
 end;

 IBProp.Attach;
 try
  IBProp.Fetch;
  GProps.Visible:=true;
  LVersion.caption:=IBProp.VersionInfo.ServerVersion;
  LImplem.caption:=IBProp.VersionInfo.ServerImplementation+
   '- SV:'+IntToStr(IBProp.Versioninfo.ServiceVersion);
  LActive.caption:=IntTostr(IBProp.DatabaseInfo.NoOfAttachments);
  LDatabases.caption:=IntTostr(IBProp.DatabaseInfo.NoOfDatabases);
  Memodb.lines.clear;
  for i:=0 to high(IBProp.DatabaseInfo.DbName) do
  begin
   MemoDB.Lines.add(IBProp.Databaseinfo.DbName[i]);
  end;
  ELock.text:=Copy(IBProp.ConfigParams.LockFileLocation,1,Length(IBProp.ConfigParams.LockFileLocation)-1);
  EMessage.text:=Copy(IBProp.ConfigParams.MessageFileLocation,1,Length(IBProp.ConfigParams.MessageFileLocation)-1);
  ESec.text:=Copy(IBProp.ConfigParams.SecurityDatabaseLocation,1,Length(IBProp.ConfigParams.SecurityDatabaseLocation)-9);
  ERoot.text:=Copy(IBProp.ConfigParams.BaseLocation,1,Length(IBProp.ConfigParams.BaseLocation)-1);
  MemoConfig.lines.clear;
  for i:=0 to high(IBProp.Configparams.ConfigFiledata.ConfigFileKey) do
  begin
   case IBProp.Configparams.ConfigFiledata.ConfigFileKey[i] of
    ISCCFG_LOCKMEM_KEY:
     astring:='LOCKMEM';
    ISCCFG_LOCKSEM_KEY:
     astring:='LOCKSEM';
    ISCCFG_LOCKSIG_KEY:
     astring:='LOCKSIG';
    ISCCFG_EVNTMEM_KEY:
     astring:='EVNTMEM';
    ISCCFG_DBCACHE_KEY:
     astring:='DBCACHE';
    ISCCFG_PRIORITY_KEY:
     astring:='PRIORITY';
    ISCCFG_IPCMAP_KEY:
     astring:='IPCMAP';
    ISCCFG_MEMMIN_KEY:
     astring:='MEMMIN';
    ISCCFG_MEMMAX_KEY:
     astring:='MEMMAX';
    ISCCFG_LOCKORDER_KEY:
     astring:='LOCKORDER';
    ISCCFG_ANYLOCKMEM_KEY:
     astring:='ANYLOCKMEM';
    ISCCFG_ANYLOCKSEM_KEY:
     astring:='ANYLOCKSEM';
    ISCCFG_ANYLOCKSIG_KEY:
     astring:='ANYLOCKSIG';
    ISCCFG_ANYEVNTMEM_KEY:
     astring:='ANYEVNTMEM';
    ISCCFG_LOCKHASH_KEY:
     astring:='LOCKHASH';
    ISCCFG_DEADLOCK_KEY:
     astring:='DEADLOCK';
    ISCCFG_LOCKSPIN_KEY:
     astring:='LOCKSPIN';
    ISCCFG_CONN_TIMEOUT_KEY:
     astring:='CON_TIMEOUT';
    ISCCFG_DUMMY_INTRVL_KEY:
     astring:='DUMMY_INTRVL';
    else
     astring:='UNKNOWN';
   end;
   MemoConfig.lines.add(astring+'='+intToStr(IBProp.Configparams.ConfigFiledata.ConfigFileValue[i]));
  end;
  for i:=0 to high(IBProp.Configparams.ConfigFileParams) do
  begin
   MemoConfig.lines.add(IBProp.Configparams.ConfigFileparams[i]);
  end;
 finally
  try
   IBProp.Detach;
  except
  end;
 end;
end;

procedure TFServerProps.BLogClick(Sender: TObject);
begin
  IBLog.Params.clear;
  IBLog.params.add('user_name='+EUsername.text);
  IBLog.params.add('password='+EPassword.text);
  IBLog.ServerName:=Trim(EServer.Text);
  if Length(IBLog.ServerName)=0 then
  begin
   IBLog.protocol:=Local;
  end
  else
  begin
   IBLog.protocol:=TProtocol(ComboProtocol.itemindex);
  end;

  IBLog.Attach;
  try
   IBLog.ServiceStart;
   ibproces.COntrolIBService(IBLog);
  finally
   try
    IBLog.Detach;
   except
   end;
  end;
 end;
end.
